/* This makes the dataset for the Gibbs sampler, which is run by Model.do. */

clear all
macro drop _all
version 12.1
set more off
set matsize 5000
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
global Name CreateDataset
capture log close

timer on 1

/* Define the programs that will be used in this .do file. */
do ${Code}/CreateDatasetPrograms.do
TimeString /* Generate the time stamp global macro. */
local time_string `r(time_string)'
log using ${Code}/Logs/${Name}_`time_string', replace

/* Make the provider id and provider zip code plan type share/treatment intensity
variables. In SetUp.do these are merged into the final dataset. */	
/* FindProvPlanTypeFraction */
FindProvIntensity
clear

/* Find the employer plan type shares. This will also be merged into the final dataset 
in Model.do. */
FindPlantypSharesByEmp
clear

/* Create a dataset which, for each person, says when they first and last appear in 
the data. They will be used to define indicators MiL6 and MiN6 later. */
CreateFirstLastAppear
clear	

/* Collect all years of inpatient data together. Restrict attention to heart disease and 
generate some variables. */
UseInptData

/* Define the procedure code global macros and categorise cardiac procedures using the 
info available in the inpatient data sets (the ccaei files). Will later refine this with 
the service data. The procedure macros will be used in GetSomeInfoFromServicesData too. */
DefineProcMacros
CatProcsUsingInptData

/* Some people have multiple records in a single admisssion date (admdate). Collapse these 
into one record. Keep the record which has the most intensive procedure (i.e. with the 
highest value of Procedure.). Change the totpay variable for this record to equal the sum 
of totpay for all records associated with that day. */
CollapseOnePerPerson
save ${Data}/EstSamplePrelim0, replace

/* Define the variable ChangeBefAdm, which indicates if a person was in sample for the 6 
months before admission and changed plan type in that time. */
local MakeChangersDta 1 /* If already have made Changers.dta from a previous run, may want 
	to not reconstruct it to save time. */
DefineChangeBefAdm `MakeChangersDta'

/* Subset the services file to include only the heart related conditions of the people in 
the sample. Merge some variables from the services file back into the inpatient admissions 
file. Change some procedure classifications on the basis of the service data. */
GetSomeInfoFromServicesData

/* Generate the cardiac procedure history variables. */
GenerateProcHistoryVars

/* Create indicators for other Elixhauser comorbidities, and some other diagnoses. */
CreateComorbidityVars

/* Define some labels, and destring some variables. */
qui do ${Code}/DefineLabels.do

save ${Data}/EstSamplePrelim1, replace

/* Get provider data, for the provider's average inpatient expenditure and length of stay, 
and for whether it is never observed to use a particular procedure. */
ProvData

/* Select the observations for the estimation routine and create ExpCov and the county 
level variables. */
SelectSampleGenVars

/* Divides the sample into low, medium and high concentration providers, as indicated by 
the variable HHI_Thirds.*/
HHI 

sort enrolid caseid	

timer off 1
timer list
save ${Data}/EstSample, replace /* This is the sample used for 
	estimation. */

exit
log close
