/* Programs used in the construction of the dataset. */

program  drop _all

program TimeString, rclass
	local c_date = c(current_date)
	local c_time = c(current_time)
	local c_time_date = "`c_date'"+"_" +"`c_time'"
	local time_string = subinstr("`c_time_date'", ":", "_", .)
	local time_string = subinstr("`time_string'", " ", "_", .)	
	return local time_string `time_string'
end

/* Find average fraction of HMO patients for providers, by id and zipcode. */
program define FindProvPlanTypeFraction
	use ${Data}/2002/ccaes2002, clear
	forvalues ii = 2003/2007{
		append using ${Data}/`ii'/ccaes`ii'
	}

	/* Generate summaries by provider. Note that the providers may be facilities as well 
	as physicians. */
	preserve
		/* Keep one observation per provid in each admission. */	
		drop if mi(provid)
		bys enrolid year caseid provid (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		gen HMO = plantyp == 4
		gen POS = plantyp == 5
		collapse (count) NumProvidAdms = HMO (mean) FracProvidHMO = HMO ///
			(mean) FracProvidPOS = POS, by(provid)
		keep provid NumProvidAdms FracProvidHMO FracProvidPOS
		save ${Data}/FracProvidPlantyp, replace
	restore

	/* Generate summaries by hospital identifier. */
	preserve
		/* Keep one observation per unihosp in each admission. */	
		drop if mi(unihosp)
		bys enrolid year caseid unihosp (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		gen HMO = plantyp == 4
		gen POS = plantyp == 5
		collapse (count) NumUnihospAdms = HMO (mean) FracUnihospHMO = HMO ///
			(mean) FracUnihospPOS = POS, by(unihosp)
		keep unihosp NumUnihospAdms FracUnihospHMO FracUnihospPOS
		save ${Data}/FracUnihospPlantyp, replace
	restore
	
	/* Generate summaries by provider zip code. */
	preserve
	/* Keep one observation per provzip in each admission. */	
		drop if mi(provzip)
		bys enrolid year caseid provzip (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		gen HMO = plantyp == 4
		gen POS = plantyp == 5
		collapse (count) NumProvzipAdms = HMO (mean) FracProvzipHMO = HMO ///
			(mean) FracProvzipPOS = POS, by(provzip)
		keep provzip NumProvzipAdms FracProvzipHMO FracProvzipPOS
		save ${Data}/FracProvzipPlantyp, replace
	restore
end

program define FindProvIntensity
	forvalues ii = 2002/2007{
		use ${Data}/`ii'/ccaes`ii', clear
		keep enrolid year caseid stdprov provid provzip seqnum
		drop if mi(provid) & mi(provzip)
		/* Don't keep the unmatched observations in the ccaei files. They almost all have 
		missing physid's anyway, so they have almost no useful information. */
		merge m:1 enrolid year caseid using ${Data}/`ii'/ccaei`ii', keep(match) /// 
			keepusing(totpay days plantyp physid unihosp) nogenerate
		compress
		save ${Data}/`ii'/Intensity`ii', replace
	}

	use ${Data}/2002/Intensity2002, clear
	forvalues ii = 2003/2007{
		append using ${Data}/`ii'/Intensity`ii'
		rm ${Data}/`ii'/Intensity`ii'.dta
	}
	
	save ${Data}/Intensity, replace

	preserve
		/* Keep one observation per provid in each admission. */	
		drop if mi(provid)
		/* isid enrolid year caseid provid seqnum, missok */
		bys enrolid year caseid provid (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		/* Find means/medians by provider. */
		collapse (count) NumProvidAdms = days (mean) MnProvidAdmLgth = days ///
			MnProvidPay = totpay (median) MdProvidAdmLgth = days ///
			MdProvidPay = totpay , by(provid)
		xtile MnProvidPayPtile = MnProvidPay, nq(100)	 /* Find percentiles of these 
				variables too. */
		xtile MdProvidPayPtile = MdProvidPay, nq(100)
		xtile MnProvidAdmLgthPtile = MnProvidAdmLgth, nq(100)
		xtile MdProvidAdmLgthPtile = MdProvidAdmLgth, nq(100)
		save ${Data}/ProvidData, replace
	restore

	preserve
		/* Keep one observation per unihosp in each admission. */	
		drop if mi(unihosp)
		/* isid enrolid year caseid unihosp seqnum, missok */
		bys enrolid year caseid unihosp (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		/* Find means/medians by provider. */
		collapse (count) NumUnihospAdms = days (mean) MnUnihospAdmLgth = days ///
			MnUnihospPay = totpay (median) MdUnihospAdmLgth = days ///
			MdUnihospPay = totpay , by(unihosp)
		xtile MnUnihospPayPtile = MnUnihospPay, nq(100)
		xtile MdUnihospPayPtile = MdUnihospPay, nq(100)
		xtile MnUnihospAdmLgthPtile = MnUnihospAdmLgth, nq(100)
		xtile MdUnihospAdmLgthPtile = MdUnihospAdmLgth, nq(100)
		save ${Data}/UnihospData, replace
	restore

	preserve
		/* Keep one observation per provzip in each admission. */	
		drop if mi(provzip)
		/* isid enrolid year caseid provzip seqnum, missok */
		bys enrolid year caseid provzip (seqnum): gen Keep = _n == 1
		keep if Keep
		drop Keep
		/* Find means/medians by provider. */
		collapse (count) NumProvzipAdms = days (mean) MnProvzipAdmLgth = days ///
			MnProvzipPay = totpay (median) MdProvzipAdmLgth = days ///
			MdProvzipPay = totpay , by(provzip)
		xtile MnProvzipPayPtile = MnProvzipPay, nq(100)
		xtile MdProvzipPayPtile = MdProvzipPay, nq(100)
		xtile MnProvzipAdmLgthPtile = MnProvzipAdmLgth, nq(100)
		xtile MdProvzipAdmLgthPtile = MdProvzipAdmLgth, nq(100)
		save ${Data}/ProvzipData, replace
	restore
end

/* For each employer, find the share of its employees in each of the plan types. */
program define FindPlantypSharesByEmp
	/* Use the monthly enrollment tables. Would use annual enrollment tables but these are 
	not available pre-2006. */
	forvalues ii = 2002/2007{
		use ${Data}/`ii'/ccaet_all_`ii', clear /* These files are converted from the 
			source datasets by ConvertMisc.stc. */
		assert ~mi(empind)
		drop if mi(plantyp)
		isid enrolid seqnum
		bys enrolid (seqnum): gen Keep = (_n == 1)
		keep if Keep
		keep plantyp empind
		xi i.plantyp, noomit
		rename _Iplantyp_* meanpt*
		gen year = `ii'
		collapse (mean) meanpt* (count) NumEnrolees = year (first) year, by(empind)
		save $Data/`ii'/PlantypShares`ii', replace
	}

	use $Data/2002/PlantypShares2002, clear
	forvalues ii = 2003/2007{
		append using $Data/`ii'/PlantypShares`ii'
	}

	mvencode meanpt*, mv(0) override /* This changes the missing values to 0 (which they 
		should be, since missing values only arise if a plan type doesn't exist in a 
		year). */
	save ${Data}/PlantypSharesByEmp, replace
end


program define CreateFirstLastAppear
	forvalues ii = 2002/2007{
		di "`ii'"
		use enrolid dtstart dtend using ${Data}/`ii'/ccaet`ii', clear
		bys enrolid (dtstart dtend): gen temp = _n == 1
		keep if temp == 1
		drop temp
		compress
		save ${Data}/`ii'/temp`ii', replace
	}	
	
	use ${Data}/2002/temp2002
	rm ${Data}/2002/temp2002.dta
	forvalues ii = 2003/2007{
		di "`ii'"
		append using ${Data}/`ii'/temp`ii'
		rm ${Data}/`ii'/temp`ii'.dta
	}
	bys enrolid: egen FirstAppear = min(dtstart)
	bys enrolid: egen LastAppear = max(dtend)
	bys enrolid: gen temp = _n == 1
	keep if temp == 1
	keep enrolid FirstAppear LastAppear
	format FirstAppear %d
	format LastAppear %d
	compress
	save ${Data}/FirstLastAppear, replace
end

/* Collect all years of inpatient data together. Restrict attention to heart disease and 
generate some variables. */

program define UseInptData
	use ${Data}/2002/ccaei2002
	forvalues ii = 2003/2007{
		append using ${Data}/`ii'/ccaei`ii'
	}
	save ${Data}/ccaei2002_2007, replace
	gen pdx_first3 = substr(pdx,1,3)
	gen pdx_first4 = substr(pdx,1,4)
	isid enrolid admdate seqnum, missok
	bys enrolid (admdate seqnum): gen Order = _n
	xtset enrolid Order
	gen Last6MthExp = 0
	sum Order if !mi(enrolid)	
	local MaxObs = r(max)
	qui forvalues ii = 1/`MaxObs'{
		replace Last6MthExp = Last6MthExp + L`ii'.totpay ///
			if admdate - L`ii'.admdate <= 183
	}
	drop Order
	keep if inlist(pdx_first3,"410","411","413","414","786") /* Keep ischemic heart 
	disease related conditions. */
	merge m:1 enrolid using ${Data}/FirstLastAppear, keep(match) 
	assert !mi(enrolid) /* Keeping only the matches from the merge above should mean all 
	mi(enrolid) are dropped. */
	gen MiL6 = admdate - 183 < FirstAppear /* Not in data for all of last 6 months. */
	gen MiN6 = admdate + 183 > LastAppear /* Not in data for all of next 6 months. */
	replace Last6MthExp = 0 if MiL6 /* Only recording spending for those who were in 
		sample for last 6 months. In model will include MiL6 as a control. */
	gen Last6MthExpThou = Last6MthExp/1000
	gen Male = sex == "1"
	drop sex
	/* Create an indicator for initial episode of care of an AMI. Have dummies 
	for each of the first four digits of pdx in the model, and one dummy for 
	the fifth. */
	gen AMI_InitEpisCare = pdx_first3 == "410" & substr(pdx,5,1) == "1"
	/* Generate a variable for another admission within a month, and an admission in the 
	subsequent 6 months. */
	bys enrolid (admdate seqnum): gen Order = _n
	xtset enrolid Order
	gen NextAdmDate = F.admdate
	gen PrevAdmDate = L.admdate
	gen CloseAdm = (NextAdmDate - admdate < 31) | (admdate - PrevAdmDate < 31) /* Since 
	missing values are +infinity this allows for NextAdmDate, PrevAdmDate to be missing. 
	*/
	cou if CloseAdm
	gen ObservedAdmNext6Mths = (NextAdmDate - admdate <= 183)
	drop Order
end


program define DefineProcMacros
  	/* Some of these codes are not for the cath itself, but indicative of a cath 
  	having being performed (e.g. the "injection procedure during cardiac 
  	catheterization" codes 93539 - ). Also omitting cath for congenital 
  	anomalies, e.g. 93530. */		
  	global CPTCathCodes 93501 93503 93508 93510 93511 93514 93524 93526 93527 ///
  		93528 93529 93539 93540 93541 93542 93543 93544 93545 93555 93556 93561 ///
  		93562 93571 93572 
  	global ICDCathCodes 8850 8851 8852 8853 8854 8855 8856 8857 8858 3721 3722 3723
  	
  	/* Including percutaneous artherectomies, but not including open angioplasties. 
  	Only including coronary angioplasties. */				
  	global CPTPTCACodes 92980 92981 92982 92984				
  	global ICDPTCACodes 0066 3601 3602 3605 3606 3607 3609 /* Including 36.06, 36.07 
  	and 36.09, as in the Dartmouth Cardiac Surgery Atlas. */
  
  	/* These CPT codes are from 2007. */
  	global CPTCABGCodes 33510 33511 33512 33513 33514 33516 33517 33518 33519 ///
  		33521 33522 33523 33530 33533 33534 33535 33536
  	global ICDCABGCodes 3610 3611 3612 3613 3614 3615 3616 3617 3619
end


program define CatProcsUsingInptData
/* Categorise cardiac procedures, using the inpatient data files. */

	gen Procedure = 1 if pproc == "" /* Non-surgical DRG assigned. See 2007 User's Guide 
	pg 49.*/
	replace Procedure = 2 if substr(pproc,1,3) == "885" | ///
	inlist(pproc,"3721","3722","3723") /* Angiocardiography or catheterization */
	replace Procedure = 3 if inlist(pproc,"0066","3601","3602","3605") /* 
	Angioplasty/Atherectomy. Note the 36.XX codes are from prior to 2005. */
	replace Procedure = 4 if substr(pproc,1,3) == "361" /* Bypass anastomosis for heart 
	revascularization */
	replace Procedure = 5 if mi(Procedure) /* Other, including other revascularization 
	(pproc beginning with 363).*/
	
	/* The rest of this section refines the procedure classification using the nonprimary 
	diagnosis codes, which can be ICD or CPT. Have checked the meaning of the codes is 
	constant from 2002-2007. Note that not reassigning patients from 
	Procedure = 5 to other procedures on the basis of the nonprimary diagnosis codes - 
	trusting MarketScan's judgement that there was some other surgery that was more 
	serious than any of the procedures 1-4 the patient may have received. */    

	forvalues ii = 1/15{
		qui destring proc`ii', gen(pc`ii') force /* This loses info on the "forced" 
		proc`ii' codes, but none of them are among the codes searched for here (since 
		all those codes are completely numeric). */
	}
	aorder
	
	/* 	Some people who receive angiocardiography according to the CPT/ICD codes are 
	misclassified according to pproc. Change Procedure accordingly. */

	{
		egen CPTCath2_15 = anymatch(pc2 - pc15), values(${CPTCathCodes})
		egen ICDCath2_15 = anymatch(pc2 - pc15), values(${ICDCathCodes})
		gen Cath2_15 = CPTCath2_15 | ICDCath2_15
		tab Procedure if Cath2_15 == 1, mi sort
		table Procedure, c(mean totpay)
		sum totpay if inlist(Procedure,1) & Cath2_15
		replace Procedure = 2 if inlist(Procedure,1) & Cath2_15
	}

	/* Some people who receive PTCA according to the CPT/ICD codes are misclassified 
	according to pproc. Change Procedure accordingly. */
	{
		egen CPTPTCA2_15 = anymatch(pc2 - pc15), values(${CPTPTCACodes})
		egen ICDPTCA2_15 = anymatch(pc2 - pc15), values(${ICDPTCACodes})
		gen PTCA2_15 = CPTPTCA2_15 | ICDPTCA2_15
		tab Procedure if PTCA2_15 == 1, mi sort
		table Procedure, c(mean totpay)
		sum totpay if inlist(Procedure,1,2) & PTCA2_15
		replace Procedure = 3 if inlist(Procedure,1,2) & PTCA2_15
	}
	
	/* Some people who receive CABG according to the CPT/ICD codes are misclassified 
	according to pproc. Change Procedure accordingly. */
	{
		egen CPTCABG2_15 = anymatch(pc2 - pc15), values(${CPTCABGCodes})
		egen ICDCABG2_15 = anymatch(pc2 - pc15), values(${ICDCABGCodes})
		gen CABG2_15 = CPTCABG2_15 | ICDCABG2_15	
		tab Procedure if CABG2_15 == 1, mi sort
		table Procedure, c(mean totpay)
		sum totpay if inlist(Procedure,1,2,3) & CABG2_15
		replace Procedure = 4 if inlist(Procedure,1,2,3) & CABG2_15
	}	
	
	/* The number of joint CABG/PTCAs. */
	cou if Procedure == 4 & PTCA2_15 == 1	
	cou
	
	/* If Procedure = 1, then the associated DRG should be non-surgical. Verify, and drop 
	cases where Procedure = 1 but DRG is surgical. The DRG codes vary from year to year. 
	*/
	tab drg if Procedure==1, mi sort
	levelsof drg if Procedure==1
	qui foreach ii in `r(levels)'{
		di "`ii'"
		tab Procedure if drg == `ii' /* To get an idea of whether these are really 
		non-surgical DRGs. */
	}

	/* Remove those observations with Procedure = 1 and DRGs which suggest catheterization 
	or surgery (checking most common drgs only - those with >= 20 appearances). 
	The codes below refer to catheterization or surgery in at least on of the years in 
	sample. Can verify this against the DRG lists that came with the medstat data (see 
	msfmt_02-07.tmproj). There seem to be few of these observations. */
	drop if Procedure == 1 & ///
		inlist(drg,124,517,516,287,518,468,109,107,249,120,251,110,547,550,983)	
end

/* Some people have multiple records in a single admisssion date (admdate). Collapse these 
into one record. Keep the record which has the most intensive procedure (i.e. with the 
highest value of Procedure.). Change the totpay variable for this record to equal the sum 
of totpay for all records associated with that day. */

program define CollapseOnePerPerson
	isid enrolid admdate Procedure seqnum, missok
	bys enrolid admdate (Procedure seqnum): egen temptotpay = sum(totpay)
	bys enrolid admdate (Procedure seqnum): egen tempphyspay = sum(physpay)
	bys enrolid admdate (Procedure seqnum): egen temphosppay = sum(hosppay)
	bys enrolid admdate (Procedure seqnum): gen tempnum = _N
	bys enrolid admdate (Procedure seqnum): gen tempcount = _n
	keep if tempnum == tempcount
	cou if tempnum > 1
	cou
	replace totpay = temptotpay if tempnum > 1
	replace physpay = tempphyspay if tempnum > 1
	replace hosppay = temphosppay if tempnum > 1
	drop temptotpay tempnum tempcount
end

/* Define the variable ChangeBefAdm, which indicates if a person was in sample for the 6 
months before admission and changed plan type in that time. */

program define DefineChangeBefAdm
	if (`1'){
	/* 	"changers.dta" will include the first plan change date recorded in the data. */
		preserve
			keep enrolid year
			duplicates drop
			save ${Data}/enrolid_year.dta, replace
			drop _all
			
			/* Extract the relevant entries in the yearly enrollment summary tables. */
			forvalues ii = 2002/2007{
				use ${Data}/`ii'/ccaet`ii'	
				gen year = `ii'
				merge m:1 enrolid year using ${Data}/enrolid_year
				/* The observations in the enrolid_year data should have corresponding 
				enrollment table entries, if the enrollment file in question is the 
				correct year. In practice this is violated in a small number of cases. */
				/* assert year != `ii' if _merge == 2  */
				keep if _merge != 1
				drop _merge
				save ${Data}/`ii'/ccaet`ii'_relevant.dta, replace
				drop _all
			}	
			
			use ${Data}/2002/ccaet2002_relevant
			forvalues ii = 2003/2007{
				append using ${Data}/`ii'/ccaet`ii'_relevant	 
			}
			save ${Data}/ccaet_relevant.dta, replace	
			
			drop if mi(plantyp) /* Don't count missing observations as plan type changes 
			*/
			/* Add the not missing clause below o/w would be ones in the first entry for 
			each enrolid */
			sort enrolid dtstart dtend 
			by enrolid: gen Change = plantyp[_n] != plantyp[_n-1] & !mi(plantyp[_n-1])
			gen ChangeDate = dtstart if Change == 1
			format %dD_m_Y ChangeDate
			keep if !mi(ChangeDate)
			sort enrolid dtstart dtend 
			/* Keep first change date */
			by enrolid: gen counter = _n
			keep if counter <= 10
			keep Change ChangeDate enrolid counter
			reshape wide ChangeDate, i(enrolid) j(counter)
			save ${Data}/changers.dta, replace
		restore	
	}
	merge m:1 enrolid using ${Data}/changers.dta, nogenerate
	replace Change = 0 if mi(Change)
	gen ChangeBefAdm = 0 /* This will be 1 for those who have changed plan type in the 6 
		months before admission. */
	forvalues ii = 1/10{
		replace ChangeBefAdm = 1 if (Change == 1) & (ChangeDate`ii' < admdate) & ///
			(ChangeDate`ii' + 183 > admdate)
	}
	replace ChangeBefAdm = 0 if MiL6
end


program define GetSomeInfoFromServicesData
{
	preserve	
		keep enrolid year caseid
		duplicates drop
		save ${Data}/enrolid_year_caseid.dta, replace
		drop _all
		
		forvalues ii = 2002/2007{
		use ${Data}/`ii'/ccaes`ii'
		merge m:1 enrolid year caseid using ${Data}/enrolid_year_caseid
		/* The observations in the enrolid_year_caseid data should have corresponding 
		services entries, if the services file in question is the correct year. */
		assert year != `ii' if _merge == 2 
		keep if _merge == 3
		drop _merge
		save ${Data}/`ii'/ccaes`ii'_relevant.dta, replace
		drop _all
		}
		
		use ${Data}/2002/ccaes2002_relevant
		forvalues ii = 2003/2007{
			append using ${Data}/`ii'/ccaes`ii'_relevant	 
		}
		

		/* Total payments, total payments for observations with missing proc1, an 
		indicator for capitation for 2007 and provider zip codes will be merged into the 
		ccaei file. Generate these variables. */
		gen pay_miproc1 = pay*mi(proc1) /* Total payments and total payments for obs with 
			missing proc1.  */
		bys enrolid year caseid (seqnum): egen tp_s = sum(pay) 	/* Sort by seqnum to avoid 
			numerical problems. The sum may depend on the order in which the variables are 	
			summed because of precision problems. Specifying (seqnum) guarantees the same 
			results each time. */
		bys enrolid year caseid (seqnum): egen tpmi_s = sum(pay_miproc1)
		gen cap = cap_svc == "Y" if year == 2007 /* Capitation indicator. cap_svc only 
			available for 2007 */	
		bys enrolid year caseid: egen somecap = max(cap)	

		bys enrolid year caseid: egen tempmodalphysid = mode(provid) ///
			if inrange(stdprov,100,799) & !mi(provid), minmode  
		bys enrolid year caseid: egen modalphysid = max(tempmodalphysid)
		bys enrolid year caseid: egen tempmodalphyszip = mode(provzip) ///
			if inrange(stdprov,100,799) & !mi(provzip), minmode
		bys enrolid year caseid: egen modalphyszip = max(tempmodalphyszip)
		
	    bys enrolid year caseid: egen tempmodalnonphysid = mode(provid) ///
	    	if !inrange(stdprov,100,799) & !mi(provid), minmode  
	    bys enrolid year caseid: egen modalnonphysid = max(tempmodalnonphysid)
	    bys enrolid year caseid: egen tempmodalnonphyszip = mode(provzip) ///
	    	if !inrange(stdprov,100,799) & !mi(provzip), minmode
	    bys enrolid year caseid: egen modalnonphyszip = max(tempmodalnonphyszip)
	
	    bys enrolid year caseid: egen tempmodalunihosp = mode(unihosp) ///
	    	if !mi(unihosp), minmode  
	    bys enrolid year caseid: egen modalunihosp = max(tempmodalunihosp)
		drop tempmodal*
		
		/* Generate an indicator for some emergency care. These codes are constant across 
		years. */
		gen Emrg = 0
		replace Emrg = 1 if inlist(revcode,"0450","0451","0452","0456","0459","0981")
		replace Emrg = 1 if inlist(stdplac,23) 
		replace Emrg = 1 if inlist(stdprov,220)
		replace Emrg = 1 if inlist(stdsvc,15,77)
		bys enrolid year caseid: egen SomeEmrg = max(Emrg)
		compress
		save ${Data}/ccaes_relevant.dta, replace
		
		/* This block inserts commas between the elements of the CPT code macros. This is 
		useful for the inlist command later. */
		local CPTCathCodesComma 
		local CPTPTCACodesComma 
		local CPTCABGCodesComma 
		foreach ii in CPTCathCodes CPTPTCACodes CPTCABGCodes{
			foreach jj in ${`ii'}{
				local `ii'Comma ``ii'Comma',`jj'
			}
		}
		
		destring proc1, gen(NumericProc1) force /* This is losing info on the non-numeric 
		codes, but they aren't used below. */
		gen CathMatch = inlist(NumericProc1`CPTCathCodesComma')
		gen PTCAMatch = inlist(NumericProc1`CPTPTCACodesComma')		
		gen CABGMatch = inlist(NumericProc1`CPTCABGCodesComma')
		
		bys enrolid year caseid: egen CathService = max(CathMatch)
		bys enrolid year caseid: egen PTCAService = max(PTCAMatch)
		bys enrolid year caseid: egen CABGService = max(CABGMatch)		
		
		keep enrolid year caseid tp_s tpmi_s somecap modalnonphysid modalnonphyszip /// 
			modalphysid modalphyszip modalunihosp SomeEmrg hospzip CathService ///
			PTCAService CABGService
		duplicates drop
		save ${Data}/SomeServicesData.dta, replace
	restore	
	merge 1:1 enrolid year caseid using ${Data}/SomeServicesData.dta, nogenerate
	tab somecap if year == 2007, mi
	cou
	replace Procedure = 2 if CathService & Procedure == 1
	replace Procedure = 3 if PTCAService & inlist(Procedure,1,2)
	replace Procedure = 4 if CABGService & inlist(Procedure,1,2,3)
	/* NB some of the "other" category includes people who have bypass/ptca. But not in 
	those categories since that's not their principal procedure (it's instead things like 
	intra-aortic balloon pumps, aneurysm excision etc.) */
}
end

program define MergeInFracProvidPlantyp
	gen provid = physid
	replace provid = modalphysid if mi(physid) /* Only want to use the service data 
		derived physician id when the inpatient data variable physid is missing. */
	replace provid = modalnonphysid if mi(physid) & mi(modalphysid) /* If both are 
		missing, use modalnonphysid. */
	merge m:1 provid using ${Data}/FracProvidPlantyp, nogenerate keep(master match) 
	
	replace unihosp = modalunihosp if mi(unihosp) /* If the inpatient data variable 
	 	unihosp is missing, then use the service data derived unihosp. */
	merge m:1 unihosp using ${Data}/FracUnihospPlantyp, nogenerate keep(master match)
	
	gen provzip = modalphyszip
	replace provzip = modalnonphyszip if mi(modalphyszip)
	replace provzip = hospzip if mi(modalphyszip) & mi(modalnonphyszip)
	merge m:1 provzip using ${Data}/FracProvzipPlantyp, nogenerate keep(master match) 

	/* Rename these variables so they aren't confused with the originals. */
	rename provid MergingProvid
	rename unihosp MergingUnihosp
	rename provzip MergingProvzip
end


/* Generate the cardiac procedure history variables. Proc`ii'_Last6Mth is 0 if the person
was not in sample for the entirety of the previous 6 months, or he was in sample for the 
previous 6 months, but didn't have the corresponding procedure in that time. It is 1 if 
the person was in sample for the previous 6 months and did have the corresponding
procedure in that time. */

program define GenerateProcHistoryVars
	assert !mi(enrolid)
	isid enrolid admdate, missok
	bys enrolid (admdate): gen Order = _n
	xtset enrolid Order
	sum Order
	local MaxObs = r(max)
	qui forvalues ii = 1/5{
		gen Proc`ii'_Last6Mth = 0
		forvalues jj = 1/`MaxObs'{
			replace Proc`ii'_Last6Mth = 1 if L`jj'.Procedure == `ii' & ///
				admdate - L`jj'.admdate <= 183
		}
		replace Proc`ii'_Last6Mth = 0 if MiL6
	}
	drop Order
end

/* Create indicators for other Elixhauser comorbidities, and some other diagnoses. */

program define CreateComorbidityVars
	do ${Code}/Elixhauser.do 
	qui forvalues ii = 1/15{
	destring dx`ii', gen(dg`ii') force
	gen dx_first3_`ii' = substr(dx`ii',1,3)
	gen dx_first4_`ii' = substr(dx`ii',1,4)
	gen dx_first5_`ii' = substr(dx`ii',1,5) /* Should be the same as the dx`ii's. */
	assert dx_first5_`ii' == dx`ii'
	destring dx_first3_`ii', gen(dg_first3_`ii') force	
	}

	qui foreach ii in $Elixhauser{
		noi di "`ii'"
		gen `ii' = 0
		forvalues jj = 3/5{
			forvalues kk = 2/15{
				foreach ll in ${`ii'_`jj'}{
					replace `ii' = 1 if dx_first`jj'_`kk' == "`ll'"
				}				
			}
		}
	}

	aorder
	egen Smoker = anymatch(dg2 - dg15), values(3051)
	egen Dysrhy = anymatch(dg_first3_2 - dg_first3_15), values(427)
	egen Cmyop = anymatch(dg_first3_2 - dg_first3_15), values(425)
	drop dg*
 	sum $Elixhauser Smoker Dysrhy Cmyop
end

/* Merge in provider intensity data. When both physician and hospital/facility data 
are available, use hospital data (because a "high intensity" physician might just be 
a physician with a high intensity speciality, e.g. a cardiac surgeon, which isn't 
really what you want to measure). Also merge in data on whether a procedure is ever 
performed by a provider, to see if this correlates with plan type (don't want people of 
one plan type going to providers who don't have the facilities to perform some 
procedure).*/
program define ProvData
	gen provid = modalnonphysid
	replace provid = physid if mi(provid)
	replace provid = modalphysid if mi(provid)
	merge m:1 provid using ${Data}/ProvidData, nogenerate keep(master match) 
	
	replace unihosp = modalunihosp if mi(unihosp) /* If the inpatient data variable 
	 	unihosp is missing, then use the service data derived unihosp. */
	merge m:1 unihosp using ${Data}/UnihospData, nogenerate keep(master match)
	
	gen provzip = modalnonphyszip
	replace provzip = hospzip if mi(provzip)
	replace provzip = modalphyszip if mi(provzip)
	merge m:1 provzip using ${Data}/ProvzipData, nogenerate keep(master match) 
	/* Rename these variables so they aren't confused with the originals. */
	rename provid MergingProvid
	rename unihosp MergingUnihosp
	rename provzip MergingProvzip
	forvalues ii = 1/5{
		foreach jj in Provid Unihosp Provzip{
			bys Merging`jj': egen Some`ii'_`jj' = max(Procedure == `ii')
			replace Some`ii'_`jj' = . if mi(Merging`jj')
		}
	}	
end


/* Select the observations for the estimation routine and create ExpCov and the county 
level variables. */
program define SelectSampleGenVars

	/* Select the observations for estimation. Define and redefine some variables used for 
	estimation.*/
	drop if mi(plantyp)
	keep if pdx_first3 == 410
	gen dropplantyp = !inlist(plantyp, 2, 4, 5, 6)
	sum dropplantyp /* NumberCode DropPT is the mean of dropplantyp. */
	drop if dropplantyp
	drop dropplantyp
	drop if region == "5"
	gen totphyspay = totpay - hosppay
	gen thpp = totphyspay/1000			
	drop if thpp <= 0	
	assert indstry != 0
	replace indstry = 0 if mi(indstry) /* Unlike the other variables, indstry has some 
		missing values. */
	replace urban = 1 if mi(urban) /* Few are mi(urban). Set to the most common value (=1) 
		if this is the case. */
	recode age (0/39 = 1) (40/44 = 2) (45/49 = 3) (50/54 = 4) (55/59 = 5) ///
		(60/max = 6), gen(agegroup)

	gen ST = pdx_first4 != 4107
	gen NoExp = (Last6MthExp <= 1)*!MiL6
	gen ExpCov = 0 /* Individuals of a given sickness probably spend different amounts in 
		different plan-types, so define ExpCov expenditure relative to others in the same 
		plan type. */

	drop if somecap == 1 /* Drop the admissions for which there is some recorded 
		capitation payment (available for 2007 only). somecap was made in
		GetSomeInfoFromServicesData. */
	sum thpp,d
	replace thpp = `r(p1)' if thpp < `r(p1)'
	replace thpp = `r(p99)' if thpp > `r(p99)'
	gen lnthpp = log(thpp)
	
	foreach ii in 2 4 5 6{
		xtile ptileExpP`ii' = Last6MthExp if Last6MthExp > 0 & plantyp == `ii', nq(100)
		replace ExpCov = ptileExpP`ii'/100 if Last6MthExp > 0 & plantyp == `ii'
	}
	
	/* Merge in the county-level data from the area resources file. */
	merge m:1 empcty using ${Data}/ARF_Data/arfclean2011.dta, ///
		keepusing(CtyName MedAgeHun MedHhldIncomeHunT IschHDMrt3YrAvgPerCapPerHun) /// 
		assert(match using) keep(match) nogen
	gen hospctynomiss = hospcty if !mi(hospcty)
	replace hospctynomiss = empcty if mi(hospcty)
	merge m:1 hospctynomiss using ${Data}/ARF_Data/arfclean2011.dta, keepusing(FracMed ///
		NumHospWAdltIntrvnCardCathPerTnT NumHospWAdltCardSrgPerTnT ///
		NumBedsCardIntnsvCarePerT NumInptSrgPerCap) assert(match using) keep(match) nogen
	foreach ii in MedAge MedHhldIncome{
		bys egeoloc: egen temp = median(`ii')
		replace `ii' = temp if mi(`ii') | `ii' == 0 /* The ARF uses 0s to signify missing 
		values. Variables may also be ==. from dividing by 0. */	
		drop temp
	}
	bys egeoloc: egen temp = median(FracMed)
	replace FracMed = temp if mi(FracMed)
	drop temp
	
	drop if mi(MnProvidAdmLgth) & mi(MnUnihospAdmLgth) & mi(MnProvzipAdmLgth) /* No 
		provider intensity data on these patients. */
end


/* This creates the HHI_Thirds variable by calling FindHHI. */
program define HHI
	FindHHI MergingUnihosp
	FindHHI MergingProvid
	FindHHI MergingProvzip
	gen HHI_Ptile = HHI_MergingUnihosp_Ptile
	replace HHI_Ptile = HHI_MergingProvid_Ptile if mi(HHI_Ptile)
	replace HHI_Ptile = HHI_MergingProvzip_Ptile if mi(HHI_Ptile)
	assert !mi(HHI_Ptile)
	xtile HHI_Thirds = HHI_Ptile, nq(3) /* Note that this is different from 
		0-33, 34-66, 67-100 of HHI_Ptile because of how that variable is constructed when 	
		variables are missing. With the 0-33... grouping don't get 1/3 of the sample in 
		each group. */
end

/* This finds the plan type HHI associated with the id variable passed to the program, and 
merges it back into the main dataset, and calculates percentiles. */
program define FindHHI
	preserve
		keep if !mi(`1')
		bys `1' plantyp: gen NumPT_ID = _N
		bys `1': gen Num_ID = _N
		gen HHISummand = 10000*((NumPT_ID/Num_ID)^2)
		bys `1' plantyp: keep if _n == 1
		bys `1' (plantyp): egen HHI_`1' = sum(HHISummand)
		bys `1': keep if _n == 1
		sum HHI_`1', d
		qui cou if HHI_`1' == 10000 /* Count providers with only one plan type. */
		local OnlyOnePT `r(N)'
		di "`r(N)' of `=_N' providers see patients from only one plan type, or	 `=`r(N)'/`=_N'*100'%"
		qui cou
		local ALL `r(N)'
		keep `1' HHI_`1'
		save ${Data}/HHI_`1', replace
	restore
	merge m:1 `1' using ${Data}/HHI_`1', nogen
	xtile HHI_`1'_Ptile = HHI_`1', nq(100)
end





