/* Estimates first stage, as described in Appendix B of paper. */
clear all
macro drop _all
version 12.1
set more off
capture log close
set matsize 5000
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
do ${Code}/${ModelFunctions}.do
set seed 100 
log using ${Code}/Logs/FirstStage_o${Otl}_bs${BSaIns}, replace

/* A. Find the most common services in the AMI data, and extract those services from the 
services data of all admissions (not just AMI). */
if (${BSaIns} == 0){
	use ${Data}/${EstSample}
	datasignature
 	keep enrolid year caseid Procedure
	merge 1:m enrolid year caseid using ${Data}/ccaes_relevant, assert(match using) /// 
		keep(match) /* ccaes_relevant is made in CreateDataset by the program 
		GetSomeInfoFromServicesData. In merged data there should be no unmatched 
		observations from the master data (EstSample). */
	bys proc1: gen Numproc1 = _N
	gen TooFewObs = Numproc1 < 500
	sum TooFewObs	/* NumberCode "Under500" is the mean of TooFewObs. */
	drop if TooFewObs | ~inlist(proctyp,"1","7") /* Only keeping those CPT/HCPCS 
		codes with a sufficient number of observations. */
	drop TooFewObs	
	levelsof proc1, clean local(ProcList)
	global ProcList `ProcList'
		forvalues ii = 2002/2007{
			use ${Data}/`ii'/ccaes`ii', clear
			gen keep = 0
			qui foreach jj of global ProcList{
				replace keep = 1 if proc1 == "`jj'"
			}
			keep if keep & inlist(plantyp,2,4,5,6) & region ~= "5"
			drop keep
			compress
			save ${Data}/`ii'/ccaes`ii'_firststage, replace
		} 
	use ${Data}/2002/ccaes2002_firststage, clear
	forvalues ii = 2003/2007{
		append using ${Data}/`ii'/ccaes`ii'_firststage	 
	}
	
	destring region, replace
	drop if cap_svc =="Y"
	isid enrolid year caseid proc1 seqnum, missok
	bys enrolid year caseid proc1 (seqnum): egen TotQty = sum(qty)
	bys enrolid year caseid proc1 (seqnum): egen TotProc1Pay = sum(pay) /* Ordering this 
		sum in case of numerical precision issues (e.g. in case the sum depends on the 
		order of the summands). */
	gen ppu = TotProc1Pay/TotQty 	
 	drop if (TotProc1Pay < 0) | ~inrange(TotQty,1,50)
	gen lnpay = log(ppu) 
	gen OON = paidntwk == "N" /* Entry recorded as paid as out of network. Only available 
		for 2007. */
	encode procmod, gen(pm) /* Procedure modifiers only available for 2007. */
	bys enrolid year caseid proc1: egen tempOON = mode(OON), minmode
	bys enrolid year caseid proc1: egen temppm = mode(pm), minmode
	drop OON pm qty pay
	rename tempOON OON
	rename temppm pm
	mvencode pm, mv(0)
	bys enrolid year caseid proc1 (seqnum): gen temp = _n
	keep if temp == 1 /* Keep one observation per admission x service. */
	drop temp	
	gen Outlier${Otl} = 0
	global MedZero
	qui foreach ii of global ProcList{
		qui sum ppu if proc1 == "`ii'",d
		if (`r(p50)' == 0){ 
			global MedZero $MedZero `ii'
			drop if proc1 == "`ii'"
		}
		else replace Outlier${Otl} = 1 if /// 	
			~inrange(ppu,`=(${Otl}^-1)*`r(p50)'',`=${Otl}*`r(p50)'') & proc1 == "`ii'"
	}
	levelsof proc1, clean local(ProcList) /* Redefine the ProcList macro. */
	global ProcList `ProcList'
	local ProcList /* Clear this macro to avoid confusion, not used in what follows. */
	compress
	save ${Data}/ccaes_firststage, replace
}

/* B. Find the service weights from the AMI data. These weights will be used by the 
RegsForFirstStage.do file called in part C below. Note that these weights 
are calculated at the individual level. So in the bootstrap they are resampled at the 
individual level. Thus the bootstrap takes into account the sampling error in calculating
the weights too. */
if (${BSaIns} == 0){
	use ${Data}/${EstSample}, clear
	keep enrolid year caseid Procedure totpay physpay hosppay
	merge 1:m enrolid year caseid using ${Data}/ccaes_firststage, keep(match) nogenerate
	/* Drop any case in which a service has an outlier. The weights
	are likely to be wrong for these cases. */
	bys enrolid year caseid: egen SomeOtl = max(Outlier${Otl})
	egen AdmissionTag = tag(enrolid year caseid)
	cou if AdmissionTag
	cou if AdmissionTag & SomeOtl
	drop if SomeOtl
	drop AdmissionTag	
	bys enrolid year caseid (seqnum): egen SumPay = sum(TotProc1Pay) 
	gen Weight = TotProc1Pay/SumPay
	drop SumPay
	drop if mi(Weight) /* Those observations with SumPay = 0. */
	bys enrolid year caseid: egen MaxWEq1 = max(Weight == 1)
	drop if MaxWEq1 /* Drop if only one procedure code accounts for all expenditure. */
	bys enrolid year caseid: egen CheckWeightSum = sum(Weight)
	assert abs(CheckWeightSum - 1) < 1e-6
	drop CheckWeightSum
	save ${Data}/EstSampleWithWeights, replace
}

/* C. Run regressions for each service and combine the estimates with the weights from
part B to generate the aIns matrix (the effect of plan type X region on procedures. */
			
	if (${BSaIns} == 0){
		use ${Data}/ccaes_firststage, clear
		cou /* NumberCode: NumObsServiceData. */
  		do ${Code}/${RegFile}.do 
		/* Define and output the insurance coefficients in the format used in 
		GSFixPriceCoefs.do (one column for each price equation). */		
		mata: RoundaIns = round(aIns, 1e-12)
	   	mata: fh = fopen("${Code}/Estimates/aIns_o${Otl}","rw")
	   	mata: fputmatrix(fh,RoundaIns)	
	   	mata: fclose(fh)	 
	}
	else{
		/* The bootstrap works as follows:
			i) Split the service data, ccaes_firststage, into two parts - one for 
			service data that corresponds to some observation in the estimation sample
			EstSample (ccaes_firststage_match), and one that does not 	
			(ccaes_firststage_nomatch)
			ii) Construct the bootstrapped service data. This is two datasets appended
				together. One is a simple clustered bootstrap sample generated from
				ccaes_firststage_nomatch. The other is a clustered bootstrap sample 
				generated from ccaes_firststage_match. The clusters in this second dataset
				are chosen by applying resampling EstSample, then choosing the 
				corresponding patients in ccaes_firststage_match.
			iii) With this new resampled dataset of services data, estimate aIns and 
				store this estimate. */
				
		/* i) */		
		use ${Data}/${EstSample}, clear
	 	keep enrolid year caseid
		merge 1:m enrolid year caseid using ${Data}/ccaes_firststage, keep(match using)
		keep enrolid year caseid proc1 egeoloc region plantyp TotQty pm lnpay ///
			Outlier${Otl} _merge
		preserve
			keep if _merge == 2 /* In "Using" data only. */
			save ${Data}/ccaes_firststage_nomatch, replace
		restore
		preserve
			keep if _merge == 3 /* In both "Using" and "Master" data. */
			save ${Data}/ccaes_firststage_match, replace
		restore
		
	 	mata: BSaIns = J(${BSReps}, 60, .) /*  For storing the bootstrap estimates of 
			aIns. 80 parameters in aIns in total (5 price equations, 4 regions, 3 plan 
			types - PPOs are omitted). */
 		forvalues RepNum = 1/35{ 
			di "Bootstrap draw number `RepNum'"
			di "Seed is `c(seed)'" /* Display seed so that restarting in the middle of the 
				run is possible. */
			timer on `RepNum'
			
			/* ii) */
			use ${Data}/ccaes_firststage_nomatch, clear
			bsample, strata(enrolid year caseid)			
			save ${Data}/BSData/ccaes_firststage_nomatch_temp, replace 
			use ${Data}/${EstSample}, clear
			bsample /* Generate the bootstrapped EstSample datasets. They will be used in 
			 	the StandardErrors program. */
			save ${Data}/BSData/BSData`RepNum', replace /* These datasets will also be
				used for in the calculation of the resampled scores - see
                StandardErrors.do. */
			keep enrolid year caseid
			bys enrolid year caseid: gen dupnum = _n
			joinby enrolid year caseid using ${Data}/ccaes_firststage /* Get observations
				from the services data corresponding to this sample. For those who 
				patients who are drawn multiple times, include all their services data for 
				each time they are drawn. */
			append using ${Data}/BSData/ccaes_firststage_nomatch_temp
			
			/* iii) */
			qui do ${Code}/${RegFile}.do
			mata: RoundVecaIns = round(vec(aIns), 1e-12)
			mata: assert(aIns == ToMatA(vec(aIns)'))
			mata: BSaIns[`RepNum',.] = RoundVecaIns'
			mata: fh = fopen("${Code}/Estimates/BSaIns_current","rw") 
			mata: fputmatrix(fh,BSaIns)	/* Save after each repetition in case of 
			 	interruption. */
			mata: fclose(fh) 
			timer off `RepNum'
			di "Time taken:"
			timer list 
		}
		mata: fh = fopen("${Code}/Estimates/BSaIns","rw")
		mata: fputmatrix(fh,BSaIns)	
		mata: fclose(fh) 
		mata: SDRoundVecaIns = diagonal(variance(BSaIns)):^.5
		mata: SDRoundaIns = ToMatA(SDRoundVecaIns)
		mata: fh = fopen("${Code}/Estimates/SDaIns_o${Otl}","rw")
		mata: fputmatrix(fh,SDRoundaIns)	
		mata: fclose(fh) 
	}
