/* May be quantity discounts, so controlling for quantity. Not including facility
relevant covariates, like stdsvc or rc here, since only looking at physician payment. 	
Omit plan type 6 (PPOs) in *each* region. It will be the base category against which 
changes	are measured. Can do this because you are including egeoloc variables, so not 
constraining different PPOs in different states to have the same intercept. */

destring egeoloc, replace
xi i.region*i.plantyp, noomit
local Cov TotQty _IregXpla_?_2 _IregXpla_?_4 _IregXpla_?_5  i.egeoloc i.pm i.year

foreach ii in 1 2 3 4{
	foreach jj in 2 4 5{
		gen Reg`ii'Plan`jj' = 0 /* These variables will contain the coefficient 
		estimates from the payment regressions. */
	}
}

/* Run the service level price regressions on the important codes. */
egen tag = tag(proc1)	
if (${BSaIns} == 0) local CurrentProcList ${ProcList}
if (${BSaIns} == 1){
	levelsof proc1, clean local(BSProcList)
	local CurrentProcList `BSProcList'
}
foreach ii in `CurrentProcList'{ 
	di "`ii'"
	qui reg lnpay `Cov' if proc1 == "`ii'" & !Outlier${Otl}
	foreach jj in 1 2 3 4{
		foreach kk in 2 4 5{
			qui replace Reg`jj'Plan`kk' = _b[_IregXpla_`jj'_`kk'] ///
				if proc1 == "`ii'" & tag
		}
	}
}

keep if tag
keep Reg* proc1

sum Reg* 

if (${BSaIns} == 0) save ${Data}/RegCoefs_o${Otl}, replace	

/* Calculate the service weights and use the regression estimates to find the effect 
of plan type on procedure prices. */

/* There shouldn't be any codes in the EstSampleWithWeights data without codes in 
RegCoefs (hence assert(match master)). But there may be some codes in RegCoefs which 
have been dropped from EstSampleWithWeights. These should be dropped in the merge 
(hence keep(match)). */
merge 1:m proc1 using ${Data}/EstSampleWithWeights, assert(match master)  keep(match)
egen VariableDropped = anymatch(Reg?Plan2 Reg?Plan4 Reg?Plan5), values(0) 
cou if VariableDropped /* In some cases, might be that you are trying to extrapolate 
how much a service would cost in some plantyp when there's no data to do so. This 
counts these observations, seems rare. */
foreach ii in 1 2 3 4{
	foreach jj in 2 4 5{
		gen WghReg`ii'Plan`jj' = Weight*Reg`ii'Plan`jj'
	}
}

/* This block of code is another way of calculating the same results. Instead of 
calculating how each person's total payment varies by plan type and then averaging 
over people, this averages the weights for each service over people, and then 
calculates how changing plan type changes the price of that average bundle of 
services. Check they are indeed the same.  */
/* 	{
preserve
	egen AdmissionTag = tag(enrolid year caseid)	
	qui forvalues ii = 1/5{
			cou if AdmissionTag & Procedure == `ii'
			local NumberPatients`ii' `r(N)'
	}


	collapse (sum) Weight (first) Reg*, by(proc1 Procedure)
	forvalues ii = 1/5{
			replace Weight = Weight/`NumberPatients`ii'' if Procedure == `ii'
			sum Weight if Procedure == `ii'
			assert float(`r(sum)') == 1
	}
	

	foreach ii in 1 2 3 4{
		foreach jj in 2 4 5 6{
			gen WghReg`ii'Plan`jj' = Weight*Reg`ii'Plan`jj'				
		}
	}
			

	collapse (sum) WghReg* Weight, by(Procedure)
	rename WghReg* p*
	list Procedure p*
restore 
} */

collapse (sum) WghReg* (first) Procedure (first) region, by(caseid enrolid year) /* 
	This gives the individual specific effect of plan type. */
collapse (mean) WghReg*, by(Procedure region) /* Averaging over individuals to get the 
average effect of plan type. */

rename WghReg?Plan? r?p?
putmata aInsData = (r1p2 r1p4 r1p5 ///
					 r2p2 r2p4 r2p5 ///
					 r3p2 r3p4 r3p5 ///
					 r4p2 r4p4 r4p5), replace /* replace option is 
	necessary for the bootstrap calculation. */
mata: aIns = GetRelevantaInsData(aInsData)'



	
