/* Called by Model. Runs the gibbs sampler for the model defined by that program. */

mata:
timer_on(1)

/* Main loop. Loop over Gibbs sampler draws. */
for(hh = 1; hh <= ngs; hh++){
	displayflush()
	/* GS PART A: Draw prices and utilities, by drawing error terms. */
	timer_on(2)
	matrixsetup(Sig, coefS = J(8, 8, .), condsdS = J(8, 1, .)) /* This function changes 
		the arguments coefS and condsdS. */
	cmlnPP = FE*aFE + WWA*aW + InsaIns	
	cmUtil = FE*bFE + WWB*bW + cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, /// 
		HHI_Thirds)
	
	ue = (uu, ee)
	/* These assertions won't be true the first time around the hh loop, hence the "hh>1" 
	condition. */
	if (hh > 1) CheckAddingUp(Util, cmUtil, ee, lnPP, cmlnPP, uu)
	for(ii=1;ii<=nobs;ii++){ 	
		ue_ii = DrawUtilErrors(hh, ue[ii, .], Procedure[ii], lnPrice[ii], FE[ii, .], /// 	
			aFE[., Procedure[ii]], WWA[ii, .], aW[., Procedure[ii]], ///
			InsaIns[ii, Procedure[ii]], cmUtil[ii, .], coefS, indmat, condsdS)
		lnPP[ii, .] = cmlnPP[ii, .] + ue_ii[., 1..5]
		if(abs(lnPP[ii, Procedure[ii]] - lnPrice[ii])>1e-8) ///
			exit(_error("Inconsistent prices."))
		Util[ii, .] = cmUtil[ii, .] + ue_ii[., 6..9] /* Utilities from procedures 2-5. */
		checkchoice(cmUtil[ii, .], ue_ii, Procedure[ii])
		uu[ii, .] = ue_ii[1..5]
		ee[ii, .] = ue_ii[6..9] 							
	}			
	timer_off(2)
	
	/* GS PART B. Draw all the price parameters. */		
	timer_on(3)
	Coefa = Sig[1..5, 6..9]*luinv(Sig[6..9, 6..9])
	Siga = Sig[1..5, 1..5] - Coefa*Sig[6..9, 1..5] /* Variance of u conditional on e. 
	Constant across people. */
	cia = cholesky(luinv(Siga))' /* cia*Siga*cia' = I. */
	CheckAddingUp(Util, cmUtil, ee, lnPP, cmlnPP, uu)	
	cmeana = (Coefa*ee')' /* Means of u conditional on e. Varies across people. */
	for(ii=1;ii<=nobs;ii++){
		AA[5*(ii-1)+1..5*ii, .] = cia*DDA[5*(ii-1)+1..5*ii, .]
		YA[5*(ii-1)+1..5*ii, .] = cia*(lnPP[ii, .]' - cmeana[ii, .]' - InsaIns[ii, .]')
	}
	/* PostVara = invFE(cross(AA, AA) + invPriorVara, cols(FE)*5, 5) */
	PostVara = luinv(cross(AA, AA) + invPriorVara)
	PostMeana = PostVara*(cross(AA, YA) + invPriorVara*PriorMeana)
	adraw = cholesky(PostVara)*rnormal(npara, 1, 0, 1) + PostMeana
	/* Compare to usual regression results. */
	/* (diagonal(luinv(AA'AA)), diagonal(PostVara))
	(luinv(AA'AA)*AA'YA, PostMeana, adraw) */
	/* Also useful to check results against the stata regression, to make 
	sure they agree with luinv(AA'AA)*AA'YY.  */	
	/* 	drop _all 
	getmata AA* = AA 
	getmata YA
	reg YA AA*, nocons */
	aW = ToMatA(adraw[1..nparanonFE])
	aFE = ToMatA(adraw[nparanonFE + 1..npara])
	timer_off(3)
			
	/* GS PART C. Draw all the utility parameters. */		
    timer_on(4)
    uu = lnPP - FE*aFE - WWA*aW - InsaIns /* Have new uu's from new aW and aIns draws. */
    cmlnPP = FE*aFE + WWA*aW + InsaIns /* Recalculate, since aW has changed. */
	cmlnPX = SubtractFirstCol(cmlnPP)
    Coefb = Sig[6..9, 1..5]*luinv(Sig[1..5, 1..5])
    Sigb = Sig[6..9, 6..9] - Coefb*Sig[1..5, 6..9] /* Variance of e conditional on u. 
	Constant across people. */
    cib = cholesky(luinv(Sigb))' /* cib*Sigb*cib' = I. */
    cmeanb = (Coefb*uu')' /* Means of e conditional on u. Varies across people. */
    for (ii = 1; ii <= nobs; ii++){
			/* Form of regressor matrix depends on how we specify bP. */
    		if (TypebP == 0) BB[4*(ii - 1) + 1..4*ii, .] = /// 
				cib*(cmlnPX[ii, .]', DDB[4*(ii - 1) + 1..4*ii, .])
    		if (TypebP == 1) BB[4*(ii - 1) + 1..4*ii, .] = ///
				cib*(cmlnPX[ii, .]'* ///
				(1 - NonResp[ii], NonResp[ii]), DDB[4*(ii - 1) + 1..4*ii, .]) 
    		if (TypebP == 2) BB[4*(ii - 1) + 1..4*ii, .] = ///
				cib*(cmlnPX[ii, .]'* ///
				(RegNum[ii]==1, RegNum[ii]==2, RegNum[ii]==3, RegNum[ii]==4), ///
				DDB[4*(ii - 1) + 1..4*ii, .]) 														
    		if (TypebP == 3){
				yearii = J(1, 6, 0) /* Hardcoding number of years here (2002-2007). */
				yearii[Year[ii] - 2002 + 1] = 1
				BB[4*(ii - 1) + 1..4*ii, .] = ///
					cib*(cmlnPX[ii, .]'*yearii, DDB[4*(ii - 1) + 1..4*ii, .])
			}
			if (TypebP == 4){
				cmlnPPBlock = (J(4, 1, -cmlnPP[ii, 1]), diag(cmlnPP[ii, 2..5])) /* This
				 	is a 4 x 5 matrix. The first column is just minus the price regressor 
					for the first treatment, and columns 2 to 5 form a diagonal matrix 
					with the other price regressors on the diagonal. */
				BB[4*(ii-1)+1..4*ii, .] = cib*(cmlnPPBlock, DDB[4*(ii-1)+1..4*ii, .])
			}	
			if (TypebP == 5){
				cmlnPPBlock = (J(4, 1, -cmlnPP[ii, 1]), cmlnPP[ii, 2..5]') /* This
				 	is a 4 x 2 matrix. The first column is just minus the price regressor 
					for the first treatment, and columns 2 is the price regressors for 
					the other treatments. */
				BB[4*(ii-1)+1..4*ii, .] = cib*(cmlnPPBlock, DDB[4*(ii-1)+1..4*ii, .])
			}
			if (TypebP == 6) BB[4*(ii - 1) + 1..4*ii, .] = ///
				cib*(cmlnPX[ii, .]'* ///
				(HHI_Thirds[ii] == 1, HHI_Thirds[ii] == 2, HHI_Thirds[ii]==3), ///
				DDB[4*(ii - 1) + 1..4*ii, .]) 														
			
    		YB[4*(ii-1)+1..4*ii, .] = cib*(Util[ii, .]' - cmeanb[ii, .]')
    }
	/* PostVarb = invFE(cross(BB, BB) + invPriorVarb, cols(FE)*4, 4)  */
	PostVarb = luinv(cross(BB, BB) + invPriorVarb)
    PostMeanb = PostVarb*(cross(BB, YB) + invPriorVarb*PriorMeanb)
    bdraw = cholesky(PostVarb)*rnormal(nparb, 1, 0, 1) + PostMeanb
    /* Compare to usual regression results. */
    /* (diagonal(luinv(BB'BB)), diagonal(PostVarb))
    (luinv(BB'BB)*BB'YB, PostMeanb, bdraw) */
    /* Also useful to check results against the stata regression, to make sure
	they agree with luinv(BB'BB)*BB'YY.  */	
    /* 	drop _all 
    getmata BB* = BB 
    getmata YB
    reg YB BB*, nocons */
    bP = bdraw[1..nparbP]
	bW = ToMatB(bdraw[nparbP+1..nparbP+nparbnonFE])
	bFE = ToMatB(bdraw[nparbP+nparbnonFE+1..nparb])
    timer_off(4)		
			
	/* GS PART D. Draw conditional variance parameters, Sig. This code is tested in 
	CheckWishartDraws.do. */
	timer_on(5)
	cmlnPP = FE*aFE + WWA*aW + InsaIns
	uu = lnPP - FE*aFE - WWA*aW - InsaIns
 	ee = Util - FE*bFE - WWB*bW - cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, ///
		Year, HHI_Thirds) 	 

	if (DiagSig == 0 & NaiveaIns == 0){ /* Unrestricted covariance. */
		Sig = DrawSig(PriorDoF, nobs, invPriorScale, ., ., uu, ee, 0)
	}
	if (DiagSig == 0 & NaiveaIns == 1){ /* Block diagonal covariance. */
		Sig = DrawSig(PriorDoF, nobs, invPriorScale, ., ., uu, ee, 1)
	}
	if (DiagSig == 1 & NaiveaIns == 0){ /* Diagonal covariance. */
		Sig = DrawSig(., nobs, ., PriorGammaAlpha, PriorGammaBeta, uu, ee, 2)
	}
	if (DiagSig == 1 & NaiveaIns == 1) ///
		exit(_error("Choose between DiagSig and NaiveaIns."))
	timer_off(5)

	/* GS PART E. Record parameters and other useful diagnostics. */
	timer_on(6)
	if(missing(Util)~=0) exit(_error("Missing Util.")) 

 	UtilRsq[hh, .] = (diagonal(variance(FE*bFE + WWB*bW + ///
		cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, HHI_Thirds))):/ ///
		diagonal(variance(Util)))' 

	lnPPRsq[hh, .] = (diagonal(variance(FE*aFE + WWA*aW + /// 
		InsaIns)):/diagonal(variance(lnPP)))'	
	Condnum[hh, .] = (cond(Sig), cond(Sig[1..5, 1..5]), cond(Sig[6..9, 6..9]))
	rescale1 = (4*Sig[6, 6])^-.5 /* Larger error variance -> unscaled parameters larger, 
		so rescaling factor smaller. */
	rescale = trace(Sig[6..9, 6..9])^-.5
	*/
	AllbW[hh, .] = vec(bW)'*rescale
	AllbWun[hh, .] = vec(bW)'
	AllbP[hh, .] = bP'*rescale
	AllbPun[hh, .] = bP'
	AllaW[hh, .] = vec(aW)'
	AllaIns[hh, .] = vec(aIns)'
	SigRscl = Sig
	SigRscl[6..9, 6..9] = (rescale^2)*SigRscl[6..9, 6..9]
	SigRscl[6..9, 1..5] = rescale*SigRscl[6..9, 1..5]
	SigRscl[1..5, 6..9] = SigRscl[6..9, 1..5]'
	AllSig[hh, .] = (vech(SigRscl)')
	AllSigun[hh, .] = (vech(Sig)')
	Checkuuee[hh, .] = mean((uu, ee))
	Allrescale[hh, .] = rescale
	Rscomp[hh, .] = (rescale1, rescale, rescale1/rescale) /* These rescalings will be 
	different, because the utility error variances are different. */		
   	AllbFE[hh, .] = vec(bFE)'*rescale
   	AllbFEun[hh, .] = vec(bFE)'
   	AllaFE[hh, .] = vec(aFE)'	

    PredUtil = FE*bFE + WWB*bW + cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, ///
		Year, HHI_Thirds) 					

    NumPred[hh] = NumPred(PredUtil, Procedure) /* How well is the model predicting 
		choices, according to the conditional means of utilities? */
	hh /* Display some output as GS progresses. */
	bP*rescale
	NumPred[hh]
   	if(mod(hh, 100)==0 | hh==1){ 
		LL[hh] = LogL(aFE, aW, aIns, bFE, bW, bP, Sig, TypebP, FE, WWA, WWB, Ins, ///
			lnPrice, Procedure, NonResp, RegNum, Year, HHI_Thirds, ReturnMean = 1, /// 
			ResetSeed = 0)	
		/* Simulate probabilities of selecting each choice. There will be some sampling 
		error in AllFraction. */
		AllFraction[hh, .] = FracProc(PredUtil, Sig)
		FracProc(PredUtil, Sig)
		FracActual
	}		
	timer_off(6)	
	
}

/* Display some output. Discarding first BurnIn draws. */

/* Autocorrelations for the first element of bP. */
acrmat = variance((AllbP[2..ngs, 1], AllbP[1..ngs-1, 1]))
acr0 = acrmat[1, 2]/(acrmat[1, 1]*acrmat[2, 2])^.5

MeanAndSDWithBurnIn(AllaFE, BurnIn, EstaFE=., SDaFE=., OutaFE=.)
MeanAndSDWithBurnIn(AllaW, BurnIn, EstaW=., SDaW=., OutaW=.)
MeanAndSDWithBurnIn(AllaIns, BurnIn, EstaIns=., SDaIns=., OutaIns=.)
MeanAndSDWithBurnIn(AllbFE, BurnIn, EstbFE=., SDbFE=., OutbFE=.)
MeanAndSDWithBurnIn(AllbW, BurnIn, EstbW=., SDbW=., OutbW=.)
MeanAndSDWithBurnIn(AllbP, BurnIn, EstbP=., SDbP=., OutbP=.)
MeanAndSDWithBurnIn(AllSig, BurnIn, EstSig=., SDSig=., OutSig=.) /* NB: Cross price 
correlations are unidentified. */

/* Find the simulated log likelihood and number of correct predictions of the estimated 
parameter vector. Need to reshape estimates. */
rshpaFE  = ToMatA(EstaFE)
rshpaW   = ToMatA(EstaW)
rshpaIns = ToMatA(EstaIns)
rshpbFE  = ToMatB(EstbFE)
rshpbW   = ToMatB(EstbW)
rshpbP   = EstbP
rshpSig  = invvech(EstSig')
eLL = LogL(rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, TypebP, ///
	FE, WWA, WWB, Ins, lnPrice, Procedure, NonResp, RegNum, Year, HHI_Thirds, ///
	ReturnMean = 1, ResetSeed = 0)	
if (ngs > 100) ix = (1\range(100, ngs, 100))

cmlnPP = FE*rshpaFE + WWA*rshpaW + InsaIns
PredEstUtil = FE*rshpbFE + WWB*rshpbW + /// 
	cmUtillnPX(TypebP, rshpbP, cmlnPP, NonResp, RegNum, Year, HHI_Thirds) 
NumEstPred = NumPred(PredEstUtil, Procedure)

Time =  c("current_time") 
if (${BSGS} == 0){
	SaveParamEstStruct(rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, ///
		TypebP, "${Code}/Estimates", "${Name}_" + Time)
	SaveParamDrawsStruct(AllaFE[ceil(BurnIn)..ngs, .], AllaW[ceil(BurnIn)..ngs, .], ///
		AllbFE[ceil(BurnIn)..ngs, .], AllbW[ceil(BurnIn)..ngs, .], ///
		AllbP[ceil(BurnIn)..ngs, .], AllSig[ceil(BurnIn)..ngs, .], ///
		"${Code}/Estimates", "${Name}_" + Time)
}
else {
	SaveParamEstStruct(rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, ///
		TypebP, "${Code}/Estimates/BootstrappedGibbsSamplerResults", /// 
		"${Name}_BS`RepNum'_OnlyaIns${OnlyaIns}")
}
	

/* mm_matlist(100*aIns, ("%8.2f", "%8.2f", "%8.2f", "%8.2f", "%8.2f"))						
fh = fopen("${Code}/Estimates/SDaIns_o${Otl}", "rw")
SDaIns = fgetmatrix(fh)
mm_matlist(100*SDaIns, ("%8.2f", "%8.2f", "%8.2f", "%8.2f", "%8.2f"))						
 */
/*
end
preserve
drop _all
mata:
aIns_U = aIns + 1.96*SDaIns
aIns_L = aIns - 1.96*SDaIns
DisplayaInsTr = 100*aIns'
DisplayaIns_UTr = 100*aIns_U'
DisplayaIns_LTr = 100*aIns_L'
end
getmata (Comp_NE HMO_NE POS_NE Comp_MW HMO_MW POS_MW ///
	Comp_S HMO_S POS_S Comp_W HMO_W POS_W) = DisplayaInsTr
getmata (Comp_NE_U HMO_NE_U POS_NE_U Comp_MW_U HMO_MW_U POS_MW_U ///
	Comp_S_U HMO_S_U POS_S_U Comp_W_U HMO_W_U POS_W_U) = DisplayaIns_UTr
getmata (Comp_NE_L HMO_NE_L POS_NE_L Comp_MW_L HMO_MW_L POS_MW_L ///
	Comp_S_L HMO_S_L POS_S_L Comp_W_L HMO_W_L POS_W_L) = DisplayaIns_LTr

gen Treatments = _n
gen PPO = 0
label define TNames 1 "Med. Mgmt" 2 "Angiography" 3 "Angioplasty" 4 "Bypass" 5 "Other", replace
label values Treatments TNames

foreach ii in NE MW S W{
if ("`ii'" == "NE") local Reg Northeast 
if ("`ii'" == "MW") local Reg Midwest
if ("`ii'" == "S") local Reg South
if ("`ii'" == "W") local Reg West
scatter HMO_`ii'_U HMO_`ii'_L ///
		POS_`ii'_U POS_`ii'_L ///
		Comp_`ii'_U Comp_`ii'_L ///
		HMO_`ii' POS_`ii' PPO Comp_`ii' ///
		Treatment, xlabel(, valuelabel labsize(medlarge)) ///
	title("`Reg'", size(huge) color(black))  ///
	connect(l l l l l l l l l l l l) ///
	lcolor(blue*.2 blue*.2 red*.2 red*.2 green*.2 green*.2 blue red black green) ///
	mcolor(blue*.2 blue*.2 red*.2 red*.2 green*.2 green*.2 blue red black green) ///
	msymbol(i i i i i i D O T S) ///
	xtitle("") ///
	ytitle("Approx. Percent Price Difference", size(medlarge)) ///
	ylabel(-10 -5 0 5) ///
	legend(order(7 "HMO" 8 "POS" 9 "PPO" 10 "Comprehensive") size(medlarge)) ///
	graphregion(color(white))
	graph export FirstStage_`ii'.pdf, replace
	graph save FirstStage_`ii', replace	
}
restore

*/				
/* mm_matlist(rshpaW,("%8.4f","%8.4f","%8.4f","%8.4f","%8.4f"),1,("dum_ad0","dum_ad1","dum_ad2","dum_ad3","dum_ad4","dum_ad5","dum_am1","dum_am2","dum_am3","dum_am4","dum_am5","dum_am6","dum_am7","dum_am8","dum_am9","dum_am10","dum_am11","dum_ag3","dum_ag4","dum_ag5","dum_ag6","dum_eg1","dum_eg4","dum_eg5","dum_eg6","dum_eg7","dum_eg8","dum_eg11","dum_eg12","dum_eg13","dum_eg16","dum_eg17","dum_eg18","dum_eg19","dum_eg20","dum_eg22","dum_eg23","dum_eg24","dum_eg25","dum_eg26","dum_eg28","dum_eg31","dum_eg32","dum_eg33","dum_eg34","dum_eg35","dum_eg36","dum_eg37","dum_eg38","dum_eg39","dum_eg41","dum_eg42","dum_eg43","dum_eg44","dum_eg46","dum_eg47","dum_eg48","dum_eg49","dum_eg52","dum_eg53","dum_eg54","dum_eg55","dum_eg56","dum_eg57","dum_eg58","dum_eg59","dum_eg61","dum_eg62","dum_eg64","dum_eg65","dum_eg100","dum_ec1","dum_ec2","dum_ec3","dum_ec4","dum_ec5","dum_ec6","dum_ec7","dum_ec8","dum_es1","dum_es2","dum_es3","dum_es4","dum_es5","dum_es6","dum_es7","dum_es8","dum_yr2002","dum_yr2003","dum_yr2004","dum_yr2005","dum_yr2006","dum_px4101","dum_px4102","dum_px4103","dum_px4104","dum_px4105","dum_px4106","dum_px4107","dum_px4108","dum_px4109","dum_iy1","dum_iy2","dum_iy3","dum_iy4","dum_iy5","dum_iy6","dum_iy7","Male","Diabetes2_15","Hypertension2_15","Smoker2_15","Obesity2_15","HeartFailure2_15","Hypothyroidism2_15","Dysrhythmias2_15","Cardiomyopathy2_15","AMI_InitEpisCare","ChangeBefAdm","MiL6","NoExp","ExpCov","Proc1_Last6Mth","Proc2_Last6Mth","Proc3_Last6Mth","Proc4_Last6Mth","Proc5_Last6Mth"),("Proc1","Proc2","Proc3","Proc4","Proc5"),"") */

timer_off(1)
timer()
end


