/* Sets up the matrices and vectors for the Gibbs sampler. */

mata:
nobs = rows(WWA)
lnPP = J(nobs, 5, 0)
cmlnPX = J(nobs, 4, 0)
Util = J(nobs, 4, 0)
uu = J(rows(WWA), 5, 0)
ee = J(rows(WWA), 4, 0)
FracActual = nobs^-1*(sum(Procedure:==1), sum(Procedure:==2), sum(Procedure:==3), ///
	sum(Procedure:==4), sum(Procedure:==5))

/* Record the (normalized) starting values. */
StartSig = Sig
StartSig[6..9, 6..9] = StartSig[6..9, 6..9]*Startrescale^2
StartSig[1..5, 6..9] = StartSig[1..5, 6..9]*Startrescale
StartSig[6..9, 1..5] = StartSig[1..5, 6..9]'
StartbW = bW*Startrescale 
StartbFE = bFE*Startrescale
StartbP = bP*Startrescale
StartaW = aW     
StartaFE = aFE   
StartaIns = aIns

/* The All* matrices will store the parameters from all the Gibbs sampler draws. */
AllbW = J(ngs, rows(bW)*4, 0)
AllbWun = AllbW
AllbFE = J(ngs, rows(bFE)*4, 0)
AllbFEun = AllbFE
AllbP = J(ngs, nparbP, 0)
AllbPun = AllbP
AllaW = J(ngs, rows(aW)*5, 0)
AllaIns = J(ngs, rows(aIns)*5, 0)
AllaFE = J(ngs, rows(aFE)*5, 0)
AllSig = J(ngs, rows(vech(Sig)), 0)
AllSigun = AllSig

/* Other information about the Gibbs sampler draws goes in these matrices. */
BurnIn = ngs/2 /* The number of draws to discard from the beginning as "burn-in". */
Checkuuee = J(ngs, 9, 0) /* Mean of residuals. */
Allrescale = J(ngs, 1, 0) /* Rescaling factors for utility. */
Rscomp = J(ngs, 3, 0) /* Compare two different rescaling factors. */
NumPred = J(ngs, 1, 0) /* Contains the number of correct treatment choice predictions. */
UtilRsq	 = J(ngs, 4, 0) /* R-squared of utility regressions. */
lnPPRsq	 = J(ngs, 5, 0) /* R-squared of log price regressions. */
LL = J(ngs, 1, 0) /* Likelihoods, as calculated by LogL(). */
Condnum = J(ngs, 3, 0) /* Condition number of variance matrix.  */
AllFraction = J(ngs, 5, 0) /* Fraction of patients getting each treatment, according to 
	the choice model. */

/* Define some matrices that are used in the regression step, which do not change
with the Gibbs sample draws (and hence can be defined outside that loop). */
cwafe = cols(WWA) + cols(FE)
cwbfe = cols(WWB) + cols(FE)
nparanonFE = cols(WWA)*5
nparbnonFE = cols(WWB)*4
npara = cwafe*5 /* Number of cond. mean parameters in price equations (excluding aIns). */
nparb = nparbP + cwbfe*4 /* Number of cond. mean parameters in utility equations. */
AA = J(5*nobs, npara, 0)
YA = J(5*nobs, 1, 0)
BB = J(4*nobs, nparb, 0)
YB = J(4*nobs, 1, 0)	
DDA = J(5*nobs, 5*cwafe, 0)
DDB = J(4*nobs, 4*cwbfe, 0)
for(ii=1;ii<=nobs;ii++){
	DDA[5*(ii-1)+1..5*ii, .] = (I(5)#WWA[ii, .], I(5)#FE[ii, .])
	DDB[4*(ii-1)+1..4*ii, .] = (I(4)#WWB[ii, .], I(4)#FE[ii, .])
}
InsaIns = Ins*aIns	/* This can be defined outside the GS loop since aIns is constant. */	

/* Define the priors. PriorDoF and PriorScaleScalar are for cond. variance and are defined 
in Model. PriorMean{a, b} and PriorVar{a, s} are for the cond. mean parameters. */
PriorScale = PriorScaleScalar*I(9) 
invPriorScale = luinv(PriorScale)
pv = 100
PriorMeana = J(npara, 1, 0)
PriorMeanb = J(nparb, 1, 0)
PriorVara = I(npara)*pv
PriorVarb = I(nparb)*pv
invPriorVara = I(npara)*pv^-1
invPriorVarb = I(nparb)*pv^-1
if (${RdForm} == 1) invPriorVarb[1, 1] = 1e4 /* bP is a scalar, since ${TypebP} == 0
 	has been asserted in SetUp.do, and it is the first element of the b vector. */
indmat = defineindmat()
end

