/* This defines inputs to the estimation routine in global macros. Then it 
passes the the appropriate variables to Mata and runs the Gibbs sampler. The Gibbs sampler 
outputs a structure which contains the estimation results. */

clear all
macro drop _all
version 12.1
set more off
set matsize 7500
set maxvar 7500
/* ssc install moremata */
set type double, perm
capture log close
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
global BSGS 0 /* This is 0 if finding the point estimates for the parameters estimated by 
Gibbs sampling (as here, and 1 if bootstrapping standard errors for those parameters (as 
in some versions of StandardErrors). */

do ${Code}/DefineGlobals.do
set seed ${seed}
log using ${Code}/Logs/${Name}, replace


do ${Code}/Elixhauser.do /* This defines the ${Elixhauser} macro. */
do ${Code}/${ModelPrograms}.do
do ${Code}/${ModelFunctions}.do
use ${Data}/${EstSample}
do ${Code}/${SetUp}.do
do ${Code}/${StartingValues}.do

/* Some summary stats. */
gen Diabetes = DwC | DwoC
sum age Male urban ${Elixhauser} Smoker Dysrhy Cmyop AMI_InitEpisCare ST SomeEmrg ///
	${CtyVars}
sum ChangeBefAdm NoExp ExpCov Last6MthExp Proc1_Last6Mth Proc2_Last6Mth Proc3_Last6Mth /// 
	Proc4_Last6Mth Proc5_Last6Mth if !MiL6
PlanTypeTmentTable /* Produces the cross tab of procedures and plantypes. */

bys plantyp: tab Procedure
tab Procedure
gen totphyspaywin = thpp*1000 /* thpp is winzorised version of totphyspay. */
gen hosppaywin = hosppay
sum hosppay, d
replace hosppaywin = `r(p99)' if hosppaywin > `r(p99)'
replace hosppaywin = `r(p1)' if hosppaywin < `r(p1)'
/* table plantyp Procedure, c(mean totpay sd totpay) row col format(%10.0fc)
table plantyp Procedure, c(mean hosppay sd hosppay) row col format(%10.0fc)
table plantyp Procedure, c(mean totphyspay sd totphyspay)  row col format(%10.0fc) */
table plantyp Procedure, c(mean totphyspaywin sd totphyspaywin)  row col format(%10.0fc)
table plantyp Procedure, c(mean hosppaywin sd hosppaywin)  row col format(%10.0fc)

/* Table 7. Number of obs by plan type and region. Point estimates and standard deviation 
of aIns. */
table plantyp region, c(n totpay) 
mata: mm_matlist(DisplayaIns(aIns), ("%8.2f", "%8.2f", "%8.2f", "%8.2f", "%8.2f"))
/* mata: fh = fopen("${Code}/Estimates/SDaIns_o${Otl}","rw")
mata: mm_matlist(DisplayaIns(fgetmatrix(fh)), ("%8.2f", "%8.2f", "%8.2f", "%8.2f", /// 
	"%8.2f"))
 */
CheckAvailability 	/* Check if there is a correlation between plan type and being seen by 
	a provider that never uses a treatment (possibly because it is not available in the
	facility the provider uses. */

do ${Code}/${InitializeGibbsSampler}.do /* This sets up some of the matrices the Gibbs 
sampler uses. */

do ${Code}/${GibbsSampler}.do

/* Calculate the marginal effects and perform some counterfactuals. */
do ${Code}/${StandardErrors}.do /* This calculates the standard error of the point 
	estimates (e.g. of bP) and also generates draws from the distribution of the parameter
	estimates which are used to calculate the standard errors of the marginal effects and 
	the counterfactual statistics. */


if ((${TypebP} == 0 | ${TypebP} == 1  | ${TypebP} == 6) & ${RdForm} == 0) ///
	do ${Code}/${MarginalEffects}.do	
/* assert the macros are correct for these counterfactuals do ${Code}/${Counterfactuals}.do  
*/
if (${TypebP} == 4) 1 - TestEqbP(AllbP, BurnIn) /* Test whether bP's, when allowed to vary 
	by procedure, are different from one another. NumberCode:*/



log close
exit
