/* This is a set of Stata programs used in Model, and the files that that file 
calls. */

/* Make crosstab of plan types and procedures. Loop through plantypes in the order: HMO, 
POS, PPO, Comprehensive. */

program define PlanTypeTmentTable
	mat PtTmTable = J(10,6,.)
	qui cou
	mat PtTmTable[10,6] = `r(N)'
	forvalues ii = 1/4{
		local pt = 4*(`ii'==1) + 5*(`ii'==2) + 6*(`ii'==3) + 2*(`ii'==4) 
		qui cou if plantyp == `pt'
		mat PtTmTable[2*`ii',6] = `r(N)'
		mat PtTmTable[2*`ii'-1,6] = 100
		forvalues jj = 1/5{
			qui cou if plantyp == `pt' & Procedure == `jj'
			mat PtTmTable[2*`ii',`jj'] = `r(N)'
			qui cou if Procedure == `jj'
			mat PtTmTable[10,`jj'] = `r(N)'
			mat PtTmTable[9,`jj'] = 100*PtTmTable[10,`jj']/PtTmTable[10,6]
			mat PtTmTable[2*(`ii'-1) + 1,`jj'] = /// 
				100*PtTmTable[2*`ii',`jj']/PtTmTable[2*`ii',6]
		}
	}
	mat PtTmTable[9,6] = 100
mat lis PtTmTable, for(%8.4g)
end

/* This program makes appropriately named dummy variables from the levels of the 
	categorical variables. */

program define genDummies
	args ii abbrev
	global Dum`ii'
	assert `ii' >= 0 & !mi(`ii')
	gen temp = 0
	qui sum `ii'
	qui forvalues jj = `r(min)'/`r(max)'{
		cou if `ii' == `jj'
	 	if `r(N)' > 0{
			gen dum_`abbrev'`jj' = `ii' == `jj'
			global Dum`ii' ${Dum`ii'} dum_`abbrev'`jj'
			replace temp = temp + dum_`abbrev'`jj'
		}
	}
	assert temp == 1 /* Check the categories are exhaustive. */
	drop temp
end
/* 
program define CfactPayProcTable
	mat CfPm = J(6,6,.)
	capture drop TrunPrice*
	qui sum thpp
	gen TrunPriceSimAct = min(max(PriceSimAct,`r(min)'),`r(max)')
	gen TrunPriceNoFin = min(max(PriceNoFin,`r(min)'),`r(max)')
	forvalues ii = 1/5{
		sum thpp if Procedure == `ii', meanonly
		mat CfPm[1,`ii'] = 1e3*`r(mean)'
		qui cou if Procedure == `ii'
		mat CfPm[2,`ii'] = 100*`r(N)'/_N
		sum TrunPriceSimAct if SimProc == `ii', meanonly
		mat CfPm[3,`ii'] = 1e3*`r(mean)'
		qui cou if SimProc == `ii'		
		mat CfPm[4,`ii'] = 100*`r(N)'/_N		
		sum TrunPriceNoFin if ProcNoFin == `ii', meanonly
		mat CfPm[5,`ii'] = 1e3*`r(mean)'
		qui cou if ProcNoFin == `ii'		
		mat CfPm[6,`ii'] = 100*`r(N)'/_N			
	}
	sum thpp, meanonly
	mat CfPm[1,6] = 1e3*`r(mean)'
	mat CfPm[2,6] = 100
	sum TrunPriceSimAct, meanonly
	mat CfPm[3,6] = 1e3*`r(mean)'
	mat CfPm[4,6] = 100
	sum TrunPriceNoFin, meanonly
	mat CfPm[5,6] = 1e3*`r(mean)'	
	mat CfPm[6,6] = 100	
	mat colnames CfPm = "MM" "Agraphy" "Aplasty" "Bypass" "Other" "Overall"
	mat rownames CfPm = "ActPay:Trnc" "ActProcFracs" "PredPay:Trnc" "PredProcFrac" /// 
		"BundledPay:Trnc" "BundledProcFrac"
	mat lis CfPm, format(%8.1fc)
end
 */
/* Calculate aIns from regressing observed log prices on controls and insurance plan type 
variables. This does not account for selection. */
program define FindNaiveaIns
global CovPlantypNoPPO RPregXpla_1_2 RPregXpla_1_4 RPregXpla_1_5 ///
				       RPregXpla_2_2 RPregXpla_2_4 RPregXpla_2_5 ///
				       RPregXpla_3_2 RPregXpla_3_4 RPregXpla_3_5 ///	
				       RPregXpla_4_2 RPregXpla_4_4 RPregXpla_4_5 
	forvalues ii = 1/5{
		qui reg lnthpp ${CovNL} ${CovFE} ${CovPlantypNoPPO} if Procedure == `ii', nocons
		mat Proc`ii'aIns = e(b)
	}
	mat aWFEIns = (Proc1aIns', Proc2aIns', Proc3aIns', Proc4aIns', Proc5aIns')
	mata: aWFEIns = st_matrix("aWFEIns")
	mata: aIns = aWFEIns[rows(aWFEIns)-11..rows(aWFEIns), .]
end

/* When both physician and hospital/facility data are available, use physician data. The
datasets used here - ProvidData, UnihospData, and ProvzipData - are made by 
FindProvIntensity. */
program define MergeInProvIntPhysFirst
	gen provid = physid
	replace provid = modalphysid if mi(provid) /* Only want to use the service data 
		derived physician id when the inpatient data variable physid is missing. */
	replace provid = modalnonphysid if mi(provid) /* If both are 
		missing, use modalnonphysid. */
	merge m:1 provid using ${Data}/ProvidData, nogenerate keep(master match) 

	replace unihosp = modalunihosp if mi(unihosp) /* If the inpatient data variable 
	 	unihosp is missing, then use the service data derived unihosp. */
	merge m:1 unihosp using ${Data}/UnihospData, nogenerate keep(master match)
	
	gen provzip = modalphyszip
	replace provzip = modalnonphyszip if mi(modalphyszip)
	replace provzip = hospzip if mi(modalphyszip) & mi(modalnonphyszip)
	merge m:1 provzip using ${Data}/ProvzipData, nogenerate keep(master match) 

	/* Rename these variables so they aren't confused with the originals. */
	rename provid MergingProvid
	rename unihosp MergingUnihosp
	rename provzip MergingProvzip
end


/* This program checks if there is a correlation between plan type and being seen by a 
provider that never uses a treatment (possibly because it is not available in the facility 
the provider uses). The Some* variables are made in ProvData. They refer to all kinds of 
heart disease, not just AMI. */
program define CheckAvailability
	qui foreach ii in Provid Unihosp Provzip{
		eststo Some3_`ii': reg Some3_`ii' b6.plantyp ${CovNL} ${CovFE}
		eststo Some4_`ii': reg Some4_`ii' b6.plantyp ${CovNL} ${CovFE}
		eststo Some5_`ii': reg Some5_`ii' b6.plantyp ${CovNL} ${CovFE}
	}	
	esttab Some3_Provid Some3_Unihosp Some3_Provzip ///
		Some4_Provid Some4_Unihosp Some4_Provzip ///
		Some5_Provid Some5_Unihosp Some5_Provzip, keep(2.plantyp 4.plantyp 5.plantyp) se
end


/* This program graphs the histogram of welfare differences between the bundled payments 
and FFS outcomes. Patient utility is an input to the program */
program define WelfareHistogram
	mata: SimOutcomesOnce(`1', cmUtil, cmlnPP, rshpSig, ProcFin = ., ///
		Paymt = ., Util1 = ., FixedSeed = 1, seed = 100)
	mata: SimOutcomesOnce(`1', cmUtilNonFin, cmlnPP, rshpSig, ProcNoFin = ., ///
		Paymt = ., Util2 = ., FixedSeed = 1, seed = 100)
	mata: cmlnPPProcNoFin = Extr(cmlnPP, ProcNoFin) /* Get prices corresponding to 
		choices. */
	mata: DollarUtilDiff = (exp(cmlnPPProcNoFin):*(exp((Util2 - Util1)/rshpbP) :- 1))*1000
	sort enrolid caseid year
	getmata ProcFin ProcNoFin DollarUtilDiff, replace
	/* tab ProcFin ProcNoFin */ /* Cross-tabulate the distribution of procedure with and 
		without financial incentives. */
	sum DollarUtilDiff if DollarUtilDiff != 0, d
	if `r(min)' >= 0{
		local low = 0
		local high = `r(p99)'*1.2
	}
	if `r(min)' < 0{
		local low = `r(p1)'*1.25
		local high = `r(max)'
	}
	hist DollarUtilDiff if inrange(DollarUtilDiff, `low', `high') & ///
		DollarUtilDiff != 0, xtitle("") graphregion(color(white)) color(blue) /// 
		lcolor(white) ytitle(,size(medlarge)) xlabel(,labsize(medlarge))
	graph save ${Code}/Estimates/`2'.gph, replace	
end







