/* This passes the matrices used in the Gibbs sampler to Mata. It is called 
by Model.do. */

		
/* Part A. Set up the globals CovNL and CovFE. Generate the variables they require. */
{
	global CovNL
	global VarList agegroup year pdx_first4 plantyp egeoloc
	foreach ii of global VarList{
		if ("`ii'" == "agegroup") 	local abbrev ag 	
		if ("`ii'" == "year") 		local abbrev yr 
		if ("`ii'" == "pdx_first4") local abbrev px 
		if ("`ii'" == "plantyp") 	local abbrev pt 
		if ("`ii'" == "egeoloc") 	local abbrev eg 		
		genDummies `ii' `abbrev' /* Makes appropriately named dummy variables from the 
			levels of the categorical variables, as well as macros collecting together the 
			dummy variables. */
	 	if ~inlist("`ii'", "plantyp", "egeoloc") global CovNL ${CovNL} ${Dum`ii'} 
	} 
	
	/* Omit one value of each variable, apart from egeoloc, to avoid collinearity. If not 
	including egeoloc, should include a constant in the model. Omit values which have a 
	reasonable amount of observations, otherwise the included dummy variables for that 
	category will be close to collinear with 1. */

	global OmittedCategories dum_ag2 dum_yr2007 dum_px4100
	global CovNL: list global(CovNL) - global(OmittedCategories)
	global CovNL ${CovNL} Male urban
	if (${ClinInfo} == 1){
		global CovNL ${CovNL} ${Elixhauser} Smoker Dysrhy Cmyop AMI_InitEpisCare ///
		ChangeBefAdm MiL6 NoExp ExpCov Proc1_Last6Mth Proc2_Last6Mth Proc3_Last6Mth /// 
		Proc4_Last6Mth Proc5_Last6Mth 
	}
	else{
		global CovNL: list global(CovNL) - global(Dumpdx_first4)
	}

	if (${CtyInfo} == 1) global CovNL ${CovNL} ${CtyVars}	/* CtyVars defined in 
		DefineGlobals.do */	
	if (${PhysInfo} == 1){
		foreach ii in Mn Md{
			foreach jj in AdmLgth AdmLgthPtile Pay PayPtile{
	/* 			gen `ii'`jj' = `ii'Provid`jj'
				replace `ii'`jj' = `ii'Unihosp`jj' if mi(`ii'`jj')
				replace `ii'`jj' = `ii'Provzip`jj' if mi(`ii'`jj') */
	 			gen `ii'`jj' = `ii'Unihosp`jj'
	  			replace `ii'`jj' = `ii'Provid`jj' if mi(`ii'`jj')
	  			replace `ii'`jj' = `ii'Provzip`jj' if mi(`ii'`jj') 
			}
		}
		cou if !mi(MnUnihospAdmLgth)
		di `r(N)'/_N /* NumberCode: ProviderFracs. */
		cou if mi(MnUnihospAdmLgth) & !mi(MnProvidAdmLgth)
		di `r(N)'/_N
		cou if mi(MnUnihospAdmLgth) & mi(MnProvidAdmLgth) & !mi(MnProvzipAdmLgth)
		di `r(N)'/_N
		gen HospPhysId = MergingUnihosp
		replace HospPhysId = MergingProvid if mi(HospPhysId)
		replace HospPhysId = MergingProvzip if mi(HospPhysId)
		codebook HospPhysId /* NumberCode: DistinctHospPhysId (see "unique values"). */
 		gen MnPayPtile1 = MnPayPtile/100
		gen MnAdmLgthPtile1 = MnAdmLgthPtile/100 
		global PhysVars MnPayPtile1 MnAdmLgthPtile1
		global CovNL ${CovNL} ${PhysVars} 
	}
	global NumPhysVars: list sizeof global(PhysVars)
	global CovFE ${Dumegeoloc}	
}

/* Part B. Set up the global CovPlantyp and (if necessary) CovPlantypRF. Generate the 
variables they require. */
{	
	/* The CovPlantyp variables will either be region interacted with actual plan type,
	or region interacted with predicted plan type, depending on the value of 
	PredIns. */

	if (${PredIns} == 0) xi i.region*i.plantyp, noomit prefix(RP)
	else{
		/* PlantypSharesByEmp is made in CreateDataset, by the program
		FindPlantypSharesByEmp. */
		merge m:1 empind year using ${Data}/PlantypSharesByEmp, ///
			keep(master match) assert(match) nogenerate
		qui forvalues ii = 1/4{
			foreach jj in 2 4 5 6{
				reg dum_pt`jj' meanpt2 meanpt4 meanpt5 meanpt6 ${CovNL} ${CovFE} ///
					if region == "`ii'"
				predict RPregXpla_`ii'_`jj' if region == "`ii'", xb	
				replace RPregXpla_`ii'_`jj' = 0 if region != "`ii'"
			}
		}
	}

	/* Important that these variables are in the same order as the rows of aIns. Goes 
	reg 1 pt 2, reg 1 pt 4,..., reg 4 pt 5. The rows corresponding to PPOs (plantyp == 6) 
	are the omitted category in each region. */
	global CovPlantyp RPregXpla_1_2 RPregXpla_1_4 RPregXpla_1_5 ///
					  RPregXpla_2_2 RPregXpla_2_4 RPregXpla_2_5 ///
					  RPregXpla_3_2 RPregXpla_3_4 RPregXpla_3_5 ///	
					  RPregXpla_4_2 RPregXpla_4_4 RPregXpla_4_5  				

	if (${RdForm} == 1){
		assert (${PredIns} == 0 & ${TypebP} == 0) /* Important bP is a scalar - see 
			InitializeGibbsSampler. */
		global CovPlantypRF ${CovPlantyp}
	}
	di "${CovNL}"
	di "${CovFE}"
}

/* Part C. Pass data to Mata and run some checks. Want to avoid "end" command since
it exits the forvalues loop, when this file is called in such a loop in StandardErrors. 
Use "mata:" prefix instead. */
{
	gen NonResp = ${NonResp}
	destring region, gen(RegNum)
	/* isid enrolid caseid year */ /* This will not be the case on the bootstrapped 
		samples.  */
	sort enrolid caseid year /* Sort on unique ordering of the data. This is important for 
	replicability. */	
	sum ${CovNL}
	putmata FE = (${CovFE}) WWA = (${CovNL}) WWB = (${CovNL} ${CovPlantypRF})  ///
		Ins = (${CovPlantyp}) lnPrice = lnthpp Procedure NonResp RegNum Year = year ///
		HHI_Thirds, replace
	mata: sum(sum(WWB))
	mata: Ins = floatround(Ins) /* If PredIns = 1, Ins is a matrix of regression predicted 
		values. Stata gives very slightly different values for these predictions each
        time, which leads to different Ins matrices for each run. This error can propagate
        in the Gibbs sampler and impede replicability. Rounding to float precision avoids
        this issue. When PredIns = 0, rounding to float precision is unnecessary but 
		changes nothing. */
 	mata: RunChecks(FE, WWA, WWB, Ins, lnPrice, Procedure, NonResp, RegNum, Year, ///
	 	HHI_Thirds,	"${CovNL}", "${CovFE}",	${RdForm})
 	mata: TypebP = ${TypebP}
    mata: ngs = ${ngs}
    mata: PriorDoF = ${PriorDoF}
    mata: PriorScaleScalar = ${PriorScaleScalar}
    mata: PriorGammaAlpha = 3
    mata: PriorGammaBeta = .5
    mata: DiagSig = ${DiagSig}
    mata: NaiveaIns = ${NaiveaIns}
}





