/* Gibbs sampler starting values. */
mata:

/* Set aIns (this is fixed throughout the estimation routine). */
if (${NaiveaIns} == 0){
	if (${BSGS} == 0){ /* This is true unless this file is being called by 
		StandardErrors, in which case should use the resampled aIns matrices. */
		fh = fopen("${Code}/Estimates/aIns_o${Otl}","rw")
		aIns = fgetmatrix(fh)
	}
	if (${BSGS} == 1){ 
		fh = fopen("${Code}/Estimates/BSaIns","rw") /* BSaIns is made by FirstStage.*/
		BSaIns = fgetmatrix(fh)
		VecaIns = BSaIns[`RepNum',.]' /* RepNum is defined in StandardErrors.  */
		aIns = ToMatA(VecaIns)
	}
	fclose(fh)	 
}
else{
	stata("FindNaiveaIns")
}

/* Set up matrices. If starting from the origin, these are the starting values. */
aW = J(cols(WWA),5,0) /* Dimensions are num of variables in WWA x num of procs. */
aFE = J(cols(FE),5,0)
bW = J(cols(WWB),4,0) /* Dimensions are num of variables in WWB x num of procs - 1. */
bFE = J(cols(FE),4,0)
if (TypebP == 0) nparbP = 1
if (TypebP == 1) nparbP = 2 /* NonResp is a binary variable. */
if (TypebP == 2) nparbP = 4 /* There are 4 regions. */
if (TypebP == 3) nparbP = max(Year) - min(Year) + 1 /* Number of years. */
if (TypebP == 4) nparbP = 5 /* Number of procedures. */
if (TypebP == 5) nparbP = 2 /* One parameter for MM, one for other procedures. */
if (TypebP == 6) nparbP = 3 /* One for each of low, medium and high HHI. */
bP = J(1,nparbP,0)
Sig_uu = diag((2, 1, 1, 1, 1))
Sig_ee = (.3, .1, .1, .1 \ ///
		  .1, .2, .1, .1 \ ///
		  .1, .1, .3, .1 \ ///
		  .1, .1, .1, .2) 
Sig_eu = J(4,5,0)                                   
Sig_ue = Sig_eu'                                    
Sig = (Sig_uu,Sig_ue\Sig_eu,Sig_ee)
/* Sig = Sig*.1  */
if (DiagSig == 1) Sig = I(9)*.7


/* Random starting points for conditional mean parameters. */
if (${StVals} == 1){  /* Watch out - with a sd of 0, rnormal produces matrices of 0s! */
	aW = /* rnormal(rows(aW),cols(aW),0,.5) */ 		J(rows(aW),1,(.2,0,.1,0,0))
	aFE = /* rnormal(rows(aFE),cols(aFE),.5,.5) */ 	J(rows(aFE),1,(0,.7,.9,2,1.4))
	bW = /* rnormal(rows(bW),cols(bW),0,.5) */ 		J(rows(bW),1,(.2,.1,.4,.3))
	bFE = /* rnormal(rows(bFE),cols(bFE),.5,.5) */ 	J(rows(bFE),1,(-.7,-.5,-4.8,-2.5))
	bP = /* rnormal(rows(bP),cols(bP),0,.5) */ 		J(rows(bP),cols(bP),.4)
	Sig_uu = diag((2,1,1,1,1))
	Sig_ee = (.3,.1,.1,.1\.1,.2,.1,.1\.1,.1,.3,.1\.1,.1,.1,.2)
	Sig_eu = J(4,5,0)                                   
	Sig_ue = Sig_eu'                                    
	Sig = (Sig_uu,Sig_ue\Sig_eu,Sig_ee)	
}



/* Use estimates from scalar bP specification as the starting values for heterogeneous
bP specification (TypebP != 0). */
  	StartingModel = "n5000t0rSomeEmrgs10pi1ci1o10d0cl1st0_06:27:35"   

if (${StVals} == 2) assert(${TypebP} != 0 | ${PhysInfo} == 0 | ${RdForm} == 1) /* NB not 
	using previous model estimates for $ClinInfo == 0, since those models are too 
	different (many fewer variables when $ClinInfo == 0). */

if (${StVals} == 2 & ${TypebP} != 0){
	GetEst(aFE = ., aW = ., bFE = ., bW = ., bPSca = ., Sig = ., InputTypebP = ., ///
		"$Code/Estimates", StartingModel)	
	assert(trace(Sig[6..9,6..9])^-.5 == 1)
	bP = J(1, cols(bP), bPSca) /* Form vector bP from scalar starting value. */
}

if (${StVals} == 2 & ${TypebP} == 0 & (${PhysInfo} == 0)){

	/* Set the starting values of the parameters which are the same dimension as the model 
	that has already been estimated (everything except aW and bW, since the previous model
	differs only because it didn't contain PhysVars as covariates). */	
	GetEst(aFE = ., aWPre = ., bFE = ., bWPre = ., bP = ., Sig = ., InputTypebP = ., ///
		"$Code/Estimates", StartingModel)	
	/* Set the values of aW and bW. */
	aW = aWPre[1..rows(aW), .] /* This depends on the physician info variables being last 
		in aWPre. */
	bW = bWPre[1..rows(bW), .]
	assert(trace(Sig[6..9,6..9])^-.5 == 1)
	assert(cols(bP) == 1)
}

/* The starting values aren't for bW and bFE in utility for the reduced form model aren't 
just the bW and bFE from the starting model - need account for the fact that W's entered 
into utilities through prices in that model too, i.e. add in the price coefficients to the 
bFE and bW here. */
if (${StVals} == 2 & ${RdForm} == 1){
	GetEst(aFE = ., aW = ., bFE = ., bW = ., bP = ., Sig = ., InputTypebP = ., ///
		"$Code/Estimates", StartingModel)	
	DiffaFE = SubtractFirstCol(aFE)
	DiffaW = SubtractFirstCol(aW)
	DiffaIns = SubtractFirstCol(aIns)
	bFE = bFE + bP*DiffaFE
	bWNonIns = bW + bP*DiffaW	
	bWIns = bP*DiffaIns
	bW = (bWNonIns \ bWIns)
	bP = 0
}
Startrescale = trace(Sig[6..9,6..9])^-.5

end

