/* This is to get some sense of the variation in the treatment payments by region. Save 
the service price coefficients corresponding to the region indicators, and use them with 
the service weights to find how the treatment payments vary by region. Service quantities 
are fixed, i.e. the only reason the treatment payments are changing across regions is 
because of the different service prices. */

clear all
macro drop _all
version 12.1
set more off
capture log close
set matsize 5000
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
do ${Code}/${ModelFunctions}.do
set seed 100

use ${Data}/ccaes_firststage, clear

destring egeoloc, replace
xi i.region*i.plantyp, noomit
local Cov TotQty _IregXpla_?_2 _IregXpla_?_4 _IregXpla_?_5  b3.region i.pm i.year

foreach ii in 1 2 3 4{
	gen Reg`ii' = . /* These variables will contain the coefficient 
		estimates from the payment regressions. */
}

/* Run the service level price regressions on the important codes. */
egen tag = tag(proc1)	
levelsof proc1, clean local(CurrentProcList)
foreach ii in `CurrentProcList'{
	di "`ii'"
	/* qui */ reg lnpay `Cov' if proc1 == "`ii'" & !Outlier${Otl}
	foreach jj in 1 2 3 4{
		qui replace Reg`jj' = _b[`jj'.region] if proc1 == "`ii'" & tag
	}
}

keep if tag
keep Reg* proc1

sum Reg* /* Make sure that these variables don't take very extreme values. */

save ${Data}/CrossRegionComparisonCoefs, replace	

/* Calculate the service weights and use the regression estimates to find the effect 
of plan type on procedure prices. */

/* There shouldn't be any codes in the EstSampleWithWeights data without codes in 
RegCoefs (hence assert(match master)). But there may be some codes in RegCoefs which 
have been dropped from EstSampleWithWeights. These should be dropped in the merge 
(hence keep(match)). */
merge 1:m proc1 using ${Data}/EstSampleWithWeights, assert(match master)  keep(match)
foreach ii in 1 2 3 4{
	gen WghReg`ii' = Weight*Reg`ii'
}

collapse (sum) WghReg* (first) Procedure, by(caseid enrolid year) /* 
	This gives the individual specific effect of plan type. */
collapse (mean) WghReg*, by(Procedure) /* Averaging over individuals to get the 
average effect of plan type. */
aaa
rename WghReg?Plan? r?p?
putmata aInsData = (r1p2 r1p4 r1p5 ///
					 r2p2 r2p4 r2p5 ///
					 r3p2 r3p4 r3p5 ///
					 r4p2 r4p4 r4p5), replace /* replace option is 
	necessary for the bootstrap calculation. */
mata: aIns = GetRelevantaInsData(aInsData)'


	
	
