mata:
/*--------------------------------------------------------------------------------------*/
/* Part A. Effect of plan type on choices in the "reduced form", i.e. when plan type 
enters utility directly and not through payments.  */
/*--------------------------------------------------------------------------------------*/

/* Define the reduced form counterfactual wrapper function. */
real matrix RdForm(FE, WWB, RegNum, bFE, bW, Sig, real scalar SE, ///
struct ParamDrawsStruct scalar D){ /* Note that this takes the structure ParamDrawsStruct
	as an input, rather than SEDrawsStruct. This latter is bootstrap draws taking into 
	account the estimation error in aIns, and this is irrelevant here since bP = 0. So can 	
	use the Gibbs sampler draws directly. */
	if (SE == 0) return(RF(FE, WWB, RegNum, bFE, bW, Sig))
	if (SE == 1){
		NumReps = 50
		DrawsInd = floor(rangen(1, rows(D.DrawsaFE), NumReps)) /* This returns NumReps
			evenly-spaced numbers from 1 to the number of draws in D. */
		Store = J(NumReps, 25, .) /* 25 elements in output of RF(). */
		for(ii = 1; ii <= NumReps; ii++){
			ii
			displayflush()
			jj = DrawsInd[ii]
			Store[ii, .] = vec(RF(FE, WWB, RegNum, ToMatB(D.DrawsbFE[jj, ]), ///
				ToMatB(D.DrawsbW[jj, ]), invvech(D.DrawsSig[jj, .]')))'
		}
		sd = diagonal(variance(Store))':^.5
		return(rowshape(sd, 5)', rowshape(mean(Store), 5)') /* Not reporting mean of 
			bootstrap estimates (mean(Store)), but keep for debugging. */	
	}
}	

/* Define the main reduced form counterfactual function. */
real matrix RF(FE, WWB, RegNum, bFE, bW, Sig){
	nobs = rows(FE)
	FracProcRF = J(5,5,.)
	FO(FE*bFE + WWB*bW, J(nobs, 5, .), Sig, UL = (., .), Ch = ., Py = .) 
		/* Not finding prices here, so pass missing values where the conditional mean of
        prices should be and for truncation points UL. */
	FracProcRF[1, .] = Ch /* First row contains model predictions from actual region x 	
		plan type values. */
	for (ii = 1; ii <= 4; ii++){ /* Compute effect of putting people into different 
		plantypes. Looping over plan types. Store in FracProcRF. */
		RFIns = MakeCfactPT(nobs, ii, RegNum)
		WWRF = WWB
		WWRF[., cols(WWB) - 11 .. cols(WWB)] = RFIns
		FO(FE*bFE + WWRF*bW, J(nobs, 5, .), Sig, UL = (., .), Ch = ., Py = .) 		
		FracProcRF[ii + 1,.] = Ch
	}
	return(FracProcRF)
}

/*--------------------------------------------------------------------------------------*/
/* Part B. Bundled payments counterfactual: choices and payments.  */
/*--------------------------------------------------------------------------------------*/


real matrix NoFin(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, ///
aW, aIns, bFE, bW, bP, Sig, UL, SE, struct SEDrawsStruct scalar D){
	ChFin = J(1, 5, .)
	ChNoFin = ChFin
	PyFin = J(1, 6, .)
	PyNoFin = PyFin
    if (SE == 0){
    	NF(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
    		aFE, aW, aIns, bFE, bW, bP, Sig, UL, ChFin, ChNoFin, PyFin, PyNoFin)
    	ChoiceTab = (ChFin \ ChNoFin) 
    	PaymentTab = (PyFin \ PyNoFin)
    	return((ChoiceTab,PaymentTab))
    }					
    if (SE == 1){
    	Store = J(rows(D.DaFE), cols((ChFin, ChNoFin, PyFin, PyNoFin)), .)
    	for(ii = 1; ii <= rows(D.DaFE); ii++){
			ii
			displayflush()
		   	GetSEDrawsii(D, ii, aFEii = ., aWii = ., aInsii = ., ///
		    	bFEii = ., bWii = ., bPii = ., Sigii = .) /* Get the iith draws from 
				the draws structure. */	
    		bPii
			NF(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
    			aFEii, aWii, aInsii, bFEii, bWii, bPii, Sigii, UL, ///
    			ChFin, ChNoFin, PyFin, PyNoFin)
    		Store[ii, .] = (ChFin, PyFin, ChNoFin, PyNoFin)
    		Store[ii, .]
    	}
		SDStore = diagonal(variance(Store)):^.5
    	return(rowshape(SDStore, 2)) /* Choices in 2x5 submatrix on left, payments in 2x6 
			submatrix on right. */
    }
}

/* Output stored in ChFin, ChNoFin, PyFin, PyNoFin. */
void NF(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, aW, ///
aIns, bFE, bW, bP, Sig, UL, ChFin, ChNoFin, PyFin, PyNoFin){
    cmlnPP = FE*aFE + WWA*aW + Ins*aIns
    cmUtilNonFin = FE*bFE + WWB*bW
    cmUtil = cmUtilNonFin + cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, /// 
    	HHI_Thirds)
    /* Find outcomes (prices and procedures) with and without financial incentives, 
	compare to the actual prices and procedures. */
    FO(cmUtil, cmlnPP, Sig, UL, ChFin = ., PyFin1 = .)
    FO(cmUtilNonFin, cmlnPP, Sig, UL, ChNoFin = ., PyNoFin1 = .)
	PyFin = (PyFin1, PyFin1*ChFin')*1e3
	PyNoFin = (PyNoFin1, PyNoFin1*ChNoFin')*1e3
}

/*--------------------------------------------------------------------------------------*/
/* Part C. Bundled payments counterfactual: welfare. */
/*--------------------------------------------------------------------------------------*/

real matrix WelfareChange(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, 
aFE, aW, aIns, bFE, bW, bP, Sig, SE, struct SEDrawsStruct scalar D){
	Num = 11
	if (SE == 0){
		Out = WC(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, ///
			aW, aIns, bFE, bW, bP, Sig, Num)
		return(rowshape(Out, 3))
	}
	if (SE == 1){
		Store = J(rows(D.DaFE), Num*3, .) /* Stores FracDiff, WDiff and WDiffFrac. */
		for(ii = 1; ii <= rows(D.DaFE); ii++){
			ii
			displayflush()
		   	GetSEDrawsii(D, ii, aFEii = ., aWii = ., aInsii = ., ///
		    	bFEii = ., bWii = ., bPii = ., Sigii = .) /* Get the iith draws from 
				the draws structure. */	
			bPii
			Store[ii, .] = WC(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, /// 
				TypebP, aFEii, aWii, aInsii, bFEii, bWii, bPii, Sigii, Num)
			Store[ii, .]
		}
		SDStore = diagonal(variance(Store)):^.5
		return(rowshape(SDStore, 3))
	}
}


real matrix WC(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, aW, ///
aIns, bFE, bW, bP, Sig, Num){
    cmlnPP = FE*aFE + WWA*aW + Ins*aIns
    cmUtilNonFin = FE*bFE + WWB*bW
    cmUtil = cmUtilNonFin + cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, /// 
    	HHI_Thirds)		
	PPO_cmlnPP = FE*aFE + WWA*aW /* If everyone is in their region's PPO, then Ins 
		is all zeroes. */
	PPO_cmUtilFin = cmUtillnPX(TypebP, bP, PPO_cmlnPP, NonResp, RegNum, Year, /// 
		HHI_Thirds)
	wgh = rangen(0, 1, Num)
	FracDiff = J(1, Num, .)
	WDiff =  J(1, Num, .)
	WDiffFrac = J(1, Num, .)
	for (ii = 1; ii <= Num; ii++){
		displayflush()
		WL(cmUtilNonFin, cmUtil, cmlnPP, PPO_cmUtilFin, Sig, bP, wgh[ii], ///
			fd = ., wl = ., wlf = .)	
		FracDiff[ii]  = fd
		WDiff[ii]     = wl
		WDiffFrac[ii] = wlf
	}
	return((FracDiff, WDiff, WDiffFrac))
}

/*--------------------------------------------------------------------------------------*/
/* Part D. Regional x plan type variation. */
/*--------------------------------------------------------------------------------------*/


real matrix RegionPlanType(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
aFE, aW, aIns, bFE, bW, bP, Sig, SE, struct SEDrawsStruct scalar D){
	if (SE == 0){
		RP(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, aW, ///
		aIns, bFE, bW, bP, Sig, ProcPriceMat = .)
		return(ProcPriceMat)
	}
	if (SE == 1){
		Store = J(rows(D.DaFE), 4*4*6, .) /* 4*4*6 = Number of elements in each 
			ProcPriceMat. */
		for(ii = 1; ii <= rows(D.DaFE); ii++){
			ii
			displayflush()
		   	GetSEDrawsii(D, ii, aFEii = ., aWii = ., aInsii = ., ///
		    	bFEii = ., bWii = ., bPii = ., Sigii = .) /* Get the iith draws from 
				the draws structure. */	
			bPii
			RP(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, TypebP, /// 
				aFEii, aWii, aInsii, bFEii, bWii, bPii, Sigii, ProcPriceMat = .)
			Store[ii, .] = vec(ProcPriceMat)'
			ProcPriceMat
		}
		SDStore = diagonal(variance(Store)):^.5
		return(rowshape(SDStore, 6)') /* This is a 4*4 x 6 matrix, with the rows and 
			columns corresponding to those of ProcPriceMat. */

	}
}

 
void RP(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, aW, ///
aIns, bFE, bW, bP, Sig, ProcPriceMat){
	ProcPriceMat = J(4*4, 6, .) /* This will store the tabulation of procedures
	 	by region across regions. Rows are ordered by plan type then region. The first 
		five columns correspond to procedures and the final one to prices. */
	for (ii = 1; ii <= 4; ii++){  /* Looping over plan type.  */
		pt_Ins = MakeCfactPT(rows(FE), ii, RegNum)
		pt_cmlnPP = FE*aFE + WWA*aW + pt_Ins*aIns
		pt_PredUtil = FE*bFE + WWB*bW + ///
			cmUtillnPX(TypebP, bP, pt_cmlnPP, NonResp, RegNum, Year, HHI_Thirds)
		Reps = 50
		ProcOutput = J(Reps, 20, .)  /* 20 = number of regions * number of procedures  */
		PriceOutput = J(Reps, 4, .)  /* 4 = number of regions  */
		for (jj = 1; jj <= Reps; jj++){
			SimOutcomesOnce(pt_PredUtil*., pt_PredUtil, pt_cmlnPP, Sig, Choiceii = ., ///
				Paymtii = ., UtilPatientii = ., FixedSeed = 0) 
			ProcOutput[jj, .] = vec(BysRegTabProc(RegNum, Choiceii))'
			PriceOutput[jj, .] = MeansByX(RegNum, Paymtii)
		}
		ProcPriceMat[4*(ii-1) + 1..4*ii, .] = (rowshape(mean(ProcOutput), 5)', ///
			mean(PriceOutput)')

	}
}



end









