/* Perform counterfactuals - bundled payments, different plan types, capitation. */

clear all
macro drop _all
version 12.1
set more off
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
do ${Code}/Elixhauser.do
do ${Code}/${ModelPrograms}.do
do ${Code}/${ModelFunctions}.do 
do ${Code}/${CFFunctions}.do 
set seed 100
global Name n5000t0rSomeEmrgs10pi1ci1o10d0cl1st0_06:27:35
use ${Data}/${EstSample}
qui do ${Code}/${SetUp}.do

mata:
GetEst(rshpaFE = ., rshpaW = ., rshpbFE = ., rshpbW = ., rshpbP = ., rshpSig = ., ///
	TypebP = ., "$Code/Estimates", "$Name")
fh = fopen("${Code}/Estimates/aIns_o${Otl}","rw")
rshpaIns = fgetmatrix(fh)


/* Part A. Reduced form. */

if (${RdForm} == 1) {
	assert (abs(rshpbP) < 1e-3) /* The prior on bP is centered on 0 with a small variance, 
		so the estimated bP should be close to 0 too. Payments don't affect utility 
		directly. Ignore bP in calculating procedure fractions and their standard 
		deviations - it's essentially zero. */
 	assert (max(rowsum(WWB[., cols(WWB) - 11..cols(WWB)])) == 1 & ///
		min(rowsum(WWB[., cols(WWB) - 11..cols(WWB)])) == 0) /* Assert that the last
		12 columns of WWB have properties one would expect if they are indeed the region
		x plan type variables. Since PPOs are omitted these variables won't sum to 1, but
		their sum should be 0 or 1. */
	timer_on(1)
	RdForm(FE, WWB, RegNum, rshpbFE, rshpbW, rshpSig, SE = 0, temp = .)
	timer_off(1)
	timer()
	stata("global Name n2500t0rSomeEmrgs10pi1ci1o10d0cl1st2_01:38:48") /* This should be 
		the name the reduced form model's output is saved under.  */
	GetDraws(ddaFE = ., ddaW = ., ddbFE = ., ddbW = ., ddbP = ., ///
		ddSig = ., "${Code}/Estimates", "$Name")
	assert(mean(ddbP) == rshpbP) /* Check the bP's match. */
	fh = fopen("${Code}/Estimates/Draws" + "$Name", "rw")
	D = fgetmatrix(fh)
	timer_on(2)
	RdForm(FE, WWB, RegNum, rshpbFE, rshpbW, rshpSig, SE = 1, D)
	timer_off(2)
	timer()
}

/* Part B. Bundled payments counterfactual: choices and payments.  */

/* UL = (exp(max(lnPrice)), exp(min(lnPrice))) */ /* Set max and min prices to the 
	windzorised values of lnPrice. */
UL = (1e8,0) /* Unrestricted max and min prices. */
NoFinOut = NoFin(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
	rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, UL, ///
	SE = 0, temp = .)
ChoiceTab1 = (Fracs(Procedure) \ NoFinOut[., 1..5]) 
PaymentTab1 = ((MeansByX(Procedure, exp(lnPrice)), mean(exp(lnPrice)))*1e3 \ ///
		        NoFinOut[., 6..11])
ChoiceTab1
PaymentTab1
fh = fopen("${Code}/Estimates/SEDraws${Name}", "rw")
D = fgetmatrix(fh)
NoFinSEOut = NoFin(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
	rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, UL, ///
	SE = 1, D)
	
stata(`"display "`c(seed)'""')
assert("`c(seed)'" == "X0438fc6ac869eb04fc6f659f9cff10d700044dc4")
/* stata("set seed X0438fc6ac869eb04fc6f659f9cff10d700044dc4") */

/* Part C. Bundled payments counterfactual: welfare. Find the fraction of people who 
receive a different treatment under bundled payments than FFS. Also find their welfare 
loss in absolute terms and as a fraction of physician payment, for varying assumptions on 
how much of the non-financial part of utility goes towards covering physician costs (this 
is determined by the wgh variable. Costs are wgh * PPO_cmlnPP).  */

WCOut = WelfareChange(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 0, temp = .)

WCSEOut = WelfareChange(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, ///
rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 1, D)
mean(WCOut[1, .]) /* NumberCode: FracDiff */
WDiff = WCOut[2,.]'
wgh = rangen(0, 1, 11)

stata(`"display "`c(seed)'""')
assert("`c(seed)'" == "X836d6d0354fd7d6cfe82807242c6521700042b35")
sd = "`c(seed)'"

end
preserve
drop _all
getmata wgh WDiff, replace
gen WDiff10 = WDiff*10
gen WDiff50 = WDiff*50
line WDiff10 wgh, lwidth(medthick) lcolor(blue) || ///
	line WDiff50 wgh if WDiff50 < 10350, lcolor(red) ///
	lwidth(medthick) graphregion(color(white)) ///
	legend(label(1 "Patient/Physician Weight = 10")  ///
		   label(2 "Patient/Physician Weight = 50")) ///
	xtitle("Cost/Payment", size(medlarge)) ///
	ytitle("Patient Welfare Gain", size(medlarge)) 
graph save ${Code}/Estimates/WelfareXCostFrac.gph, replace
restore

/* For one particular set of utility error draws for the sample, create the graph of 
utility differences, for the cases where wgh = 0 and wgh = 1. */

/* WelfareHistogram cmUtilNonFin WelfareHist_a
WelfareHistogram "cmUtilNonFin + PPO_cmUtilFin" WelfareHist_b
 */

/* See if large welfare losses are linked to readmissions. */
/* gen logDollarUtilDiff = log(DollarUtilDiff)
gen SubOpt = logDollarUtilDiff > 0
reg ObservedAdmNext6Mths SubOpt ${CovNL} MiN6 i.Procedure
reg ObservedAdmNext6Mths DollarUtilDiff ${CovNL} MiN6 i.Procedure
 */
	
mata:

/* Part C. What if everyone were in their region's version of a particular
plan type? (This is somewhat like the reduced form, but Ins affects utilities in a more 
restricted way.) Breaking up by region. */
assert("`c(seed)'" == "X836d6d0354fd7d6cfe82807242c6521700042b35")
PriceProcMat = RegionPlanType(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, ///
	TypebP, rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 0, temp = .)

PriceProcSEMat = RegionPlanType(FE, WWA, WWB, NonResp, RegNum, Year, HHI_Thirds, ///
	TypebP, rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 1, D)

Ord = (5, 9, 13, 1,  ///
	   6, 10, 14, 2, ///
	   7, 11, 15, 3, ///
	   8, 12, 16, 4) /* ProcMat rows ordered by plan type and region, with plan type in 
		the order Comp, HMO, POS, PPO. Use Ord to order by region then plan type, with 
		plan type ordered as HMO, POS, PPO, Comp. */
mm_matlist((100*(PriceProcMat[Ord, 1..5]), 1000*PriceProcMat[Ord, 6]), ///
	("%8.1f","%8.1f","%8.1f","%8.1f","%8.1f","%8.0f"))
mm_matlist((100*(PriceProcSEMat[Ord, 1..5]), 1000*PriceProcSEMat[Ord, 6]), ///
	("%8.1f","%8.1f","%8.1f","%8.1f","%8.1f","%8.0f"))
	
	
/* Part D. Predicted treatments for the capitated patients. */
clear 
do ${Code}/DefineGlobals.do
qui do ${Code}/${ModelPrograms}.do
qui do ${Code}/${ModelFunctions}.do
use ${Data}/${EstSample} /* Can't run SetUp on CapSample only - it doesn't generate 
	all the necessary variables, e.g. region x plan type interactions for which there are 
	no observations in CapSample. So add and remove EstSample instead. */
gen FromEstSample = 1
append using ${Data}/CapSample.dta
replace lnthpp = 1e10 if FromEstSample != 1 /* To avoid throwing a missing data error in 
SetUp. These values are never used, since the model is not estimated on this data. */
do ${Code}/${SetUp}.do
keep if FromEstSample != 1 /* Keep the capitated patients. */
putmata FE = (${CovFE}) WWA = (${CovNL}) WWB = (${CovNL} ${CovPlantypRF})  ///
	Ins = (${CovPlantyp}) lnPrice = lnthpp Procedure NonResp RegNum Year = year ///
	HHI_Thirds, replace
tab Procedure
mata:
/* GetEst(rshpaFE = ., rshpaW = ., rshpbFE = ., rshpbW = ., rshpbP = ., rshpSig = ., 
InputTypebP = ., "$Code/Estimates", "n2000t0rSomeEmrgs888pi1ci1o10d0cl1st0_02:44:48") 
 */
FP = J(100, 5, .)
for(ii = 1; ii <= 100; ii++){ /* Fewer capitated observations so sampling error in 
FracProc calculation is larger. Average over several simulations to average out sampling 
error. */
	FP[ii, .] = FracProc(FE*rshpbFE + WWB*rshpbW, rshpSig)
}
mean(FP)

end


