/* Calculate average partial effects in the probit model. This calculates partial 
effects separately for NonResp = 1 and 0. Accommodates the cases when bP is 1x1 
and 2x1. */

clear all
macro drop _all
version 12.1
set more off
set type double, perm
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
do ${Code}/Elixhauser.do
do ${Code}/${ModelPrograms}.do
do ${Code}/${ModelFunctions}.do 
set seed 100
global Name n5000t0rSomeEmrgs10pi0ci1o10d0cl0st0_13:48:24

use ${Data}/${EstSample}
qui do ${Code}/${SetUp}.do

mata:
GetEst(rshpaFE = ., rshpaW = ., rshpbFE = ., rshpbW = ., rshpbP = ., rshpSig = ., ///
	TypebP = ., "$Code/Estimates", "$Name")
fh = fopen("${Code}/Estimates/aIns_o${Otl}","rw")
rshpaIns = fgetmatrix(fh)

mata:

/* Define the marginal effects wrapper function. */
real matrix MargEff(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, TypebP, aFE, ///
aW, aIns, bFE, bW, bP, Sig, real scalar SE, struct SEDrawsStruct scalar D){   
    cmlnPP = FE*aFE + WWA*aW + Ins*aIns
    PredUtil = FE*bFE + WWB*bW + ///
    	cmUtillnPX(TypebP, bP, cmlnPP, NonResp, RegNum, Year, HHI_Thirds)
    Vee = Sig[6..9, 6..9]
    if (SE == 0) return(ME(PredUtil, Vee, bP, TypebP, NonResp, HHI_Thirds, 0))
 	if (SE == 1){
		Draws = rows(D.DaFE)
		Store = J(Draws, 25*cols(bP), .) /* 25 is the number of elements in each APE 
			matrix, of which there are cols(bP). */
		for(ii = 1; ii <= Draws; ii++){
			ii
			displayflush()
			GetSEDrawsii(D, ii, aFEii = ., aWii = ., aInsii = ., ///
				bFEii = ., bWii = ., bPii = ., Sigii = .) /* Get the iith draws from the 
					draws structure. */
			bPii		
			cmlnPPii = FE*aFEii + WWA*aWii + Ins*aInsii
		    PredUtilii = FE*bFEii + WWB*bWii + ///
		    	cmUtillnPX(TypebP, bPii, cmlnPPii, NonResp, RegNum, Year, HHI_Thirds)
		    Veeii = Sigii[6..9, 6..9]
		    Store[ii, .] = vec(ME(PredUtilii, Veeii, bPii, TypebP, NonResp, ///
				HHI_Thirds, 1))'
			Store[ii, .]
		}
		sd = diagonal(variance(Store))':^.5
		return(rowshape(sd, 5*cols(bP))', rowshape(mean(Store), 5*cols(bP))') /* Not 
			reporting mean of bootstrap estimates (mean(Store)), but keep for debugging.*/
	}
}

/* Define the main marginal effects function. */
real matrix ME(real matrix PredUtil, real matrix Vee, real matrix bP, ///
real scalar TypebP, real matrix NonResp, real matrix HHI_Thirds, real scalar SE){

	if(TypebP != 0 & TypebP != 1 & TypebP != 6) ///
		exit(_error("Marginal effects program currently only for TypebP = 0,1 or 6."))
	nobs = rows(PredUtil)
		
	/* Define the relevant transformations. */
	c1 = J(4, 1, -1)
	c2 = (0\1\0\0)
	c3 = (0\0\1\0)
	c4 = (0\0\0\1)

	T1 = I(4)
	T2 = (c1, c2, c3, c4)
	T3 = (c2, c1, c3, c4)
	T4 = (c2, c3, c1, c4)
	T5 = (c2, c3, c4, c1)

	Trunc1 = -(T1*PredUtil')'
	Trunc2 = -(T2*PredUtil')'
	Trunc3 = -(T3*PredUtil')'
	Trunc4 = -(T4*PredUtil')'
	Trunc5 = -(T5*PredUtil')'

	TVee1 = T1*Vee*T1'
	TVee2 = T2*Vee*T2'
	TVee3 = T3*Vee*T3'
	TVee4 = T4*Vee*T4'
	TVee5 = T5*Vee*T5'
	
	/* Compute choice probabilities and derivatives observation by observation
	(seems like there is a bug in ghkfast for the vector computation - it terminates
	after a probability is evaluated to be 0?)  */
	S = ghkfast_init(nobs, 10, 4, "ghalton")
	p1 = J(nobs, 1, 0)
	p2 = p1
	p3 = p1
	p4 = p1
	p5 = p1
	d1predu = J(nobs, 4, 0)	
	d2predu = d1predu
	d3predu = d1predu
	d4predu = d1predu
	d5predu = d1predu
	dtp = J(1, 4, 0)
	dtv = J(1, cols(vech(Vee)), 0)
	for (ii = 1; ii <= nobs; ii++){ 
/* 		if (ii/1000 == floor(ii/1000)){
 			ii
			displayflush() 
		}		 
 */		p1[ii] = ghkfast_i(S, Trunc1, TVee1, ii, dtp, dtv)
		d1predu[ii, .] = dtp
		p2[ii] = ghkfast_i(S, Trunc2, TVee2, ii, dtp, dtv)
		d2predu[ii, .] = dtp
		p3[ii] = ghkfast_i(S, Trunc3, TVee3, ii, dtp, dtv)
		d3predu[ii, .] = dtp
		p4[ii] = ghkfast_i(S, Trunc4, TVee4, ii, dtp, dtv)
		d4predu[ii, .] = dtp
		p5[ii] = ghkfast_i(S, Trunc5, TVee5, ii, dtp, dtv)
		d5predu[ii, .] = dtp
	}
	
	/* assert(max(1 :- (p1+p2+p3+p4+p5))<1e-4) */ /* This will only be true if the number 
	of draws is reasonably large. Even if it's violated the APEs may be pretty close, 
	since they average over all observations. */
	if (SE == 0){
		max(1 :- (p1 + p2 + p3 + p4 + p5))
		mean(1 :- (p1 + p2 + p3 + p4 + p5))
		variance(1 :- (p1 + p2 + p3 + p4 + p5))^.5
	}

	dUdp1 = J(4, 1, -1)
	dUdp2 = (1 \ 0 \ 0 \ 0)
	dUdp3 = (0 \ 1 \ 0 \ 0)
	dUdp4 = (0 \ 0 \ 1 \ 0)
	dUdp5 = (0 \ 0 \ 0 \ 1)
	dUdp = (dUdp1, dUdp2, dUdp3, dUdp4, dUdp5)

	dUdp_bP1 = dUdp*bP[1]
	PEii_bP1 = J(5, 5, 0)
	AllPE_bP1 = J(nobs, 25, 0)
			
	if (TypebP == 1 | TypebP == 6){
		dUdp_bP2 = dUdp*bP[2]
		PEii_bP2 = J(5, 5, 0)
		AllPE_bP2 = J(nobs, 25, 0)
	}
	
	if (TypebP == 6){
		dUdp_bP3 = dUdp*bP[3]
		PEii_bP3 = J(5, 5, 0)
		AllPE_bP3 = J(nobs, 25, 0)
	}
	
	dd = (d1predu, d2predu, d3predu, d4predu, d5predu)
	TT = (T1, T2, T3, T4, T5)
	
	/* Find the matrix of partial effects. If pk changes then U changes by dUdpk*bP 
	and the transformed U for the jth procedure changes by -Tj*dUdpk, so the partial 
	effect on the jth procedure w.r.t pk is -ddj'*Tj*dUdpk. */

	for (ii=1; ii <= nobs; ii++){
		for (jj = 1; jj <= 5; jj++){ /* Procedures */
			for (kk = 1; kk <= 5; kk++){	/* Prices */
				PEii_bP1[jj, kk] = /// 
					-dd[ii, 4*(jj-1)+1..4*jj]*TT[., 4*(jj-1)+1..4*jj]*dUdp_bP1[., kk]
				if (TypebP == 1 | TypebP == 6) PEii_bP2[jj, kk] = /// 
					-dd[ii, 4*(jj-1)+1..4*jj]*TT[., 4*(jj-1)+1..4*jj]*dUdp_bP2[., kk]	
				if (TypebP == 6) 			   PEii_bP3[jj, kk] = /// 
					-dd[ii, 4*(jj-1)+1..4*jj]*TT[., 4*(jj-1)+1..4*jj]*dUdp_bP3[., kk]						
			}
		}
		AllPE_bP1[ii, .] = vec(PEii_bP1)'
		if (TypebP == 1 | TypebP == 6) AllPE_bP2[ii, .] = vec(PEii_bP2)'
		if (TypebP == 6) 			   AllPE_bP3[ii, .] = vec(PEii_bP3)'		
	}

	/* Note that the colsums of the APEmat's should be zero, because if you change the 
	price of a procedure by some amount the changes in the probabilities of all possible 
	procedures should sum to 0. The rowsums should also be zero, because APEmat is 
	symmetric. If SE == 0, run some tests to check APEs are calculated correctly (should 
	be unnecessary to verify for SE == 1 too). */
	if (TypebP == 0){
		APEvec_bP1 = mean(AllPE_bP1)
		APEmat_bP1 = rowshape(APEvec_bP1, 5)'
/* 		APEmat_bP1 = (APEvec_bP1[1..5]', APEvec_bP1[6..10]', APEvec_bP1[11..15]', /// 
			APEvec_bP1[16..20]', APEvec_bP1[21..25]')*/
		/* Calculate APEs another way and compare. */		
	    if ((SE == 0) & (abs(APEmat_bP1[1, 1] - bP*sum(mean(d1predu)) > 1e-6))) ///
	   		exit(_error("APE calculation error 1."))
	    if ((SE == 0) & (abs(APEmat_bP1[2, 3] + bP*mean(d2predu[., 2]) > 1e-6))) ///
	   		exit(_error("APE calculation error 2."))	
		APEmat_bP1
		return(APEmat_bP1)
	}
    
	if (TypebP == 1){
		APEvec_bP1 = colsum(AllPE_bP1:*(1 :- NonResp))/sum(1 :- NonResp)
		APEvec_bP2 = colsum(AllPE_bP2:*NonResp)/sum(NonResp)
		APEmat_bP1 = rowshape(APEvec_bP1, 5)'
		APEmat_bP2 = rowshape(APEvec_bP2, 5)'
		/* Calculate APEs another way and compare. */
		if ((SE == 0) & (abs(APEmat_bP2[1, 1] - bP[2]* ///
			sum(sum(d1predu:*NonResp)/sum(NonResp))) > 1e-6)) /// 	
			exit(_error("APE calculation error 4."))
		if ((SE == 0) & (abs(APEmat_bP2[2, 3] + bP[2]* ///
			sum(d2predu[., 2]:*NonResp)/sum(NonResp) > 1e-6))) /// 	
			exit(_error("APE calculation error 5."))
		if ((SE == 0) & ///
			(abs(APEmat_bP1[2, 3] + bP[1]*sum(d2predu[., 2]:*(1:-NonResp))/ /// 
			sum(1:-NonResp) > 1e-6))) exit(_error("APE calculation error 6."))
		(APEmat_bP1, APEmat_bP2)
   		return((APEmat_bP1, APEmat_bP2))
	}
	
	if (TypebP == 6){
		APEvec_bP1 = colsum(AllPE_bP1:*(HHI_Thirds :== 1))/sum(HHI_Thirds :== 1)
		APEvec_bP2 = colsum(AllPE_bP2:*(HHI_Thirds :== 2))/sum(HHI_Thirds :== 2)
		APEvec_bP3 = colsum(AllPE_bP3:*(HHI_Thirds :== 3))/sum(HHI_Thirds :== 3)
		APEmat_bP1 = rowshape(APEvec_bP1, 5)'
		APEmat_bP2 = rowshape(APEvec_bP2, 5)'
		APEmat_bP3 = rowshape(APEvec_bP3, 5)'
		/* Calculate APEs another way and compare. */
		if ((SE == 0) & (abs(APEmat_bP1[1, 1] - bP[1]* ///
			sum(sum(d1predu:*(HHI_Thirds :== 1))/sum(HHI_Thirds :== 1))) > 1e-6)) /// 	
			exit(_error("APE calculation error 7."))
		if ((SE == 0) & (abs(APEmat_bP2[2, 3] + bP[2]* ///
			sum(d2predu[., 2]:*(HHI_Thirds :== 2))/sum(HHI_Thirds :== 2) > 1e-6))) /// 	
			exit(_error("APE calculation error 8."))
		if ((SE == 0) & ///
			(abs(APEmat_bP3[2, 3] + bP[3]*sum(d2predu[., 2]:*(HHI_Thirds :== 3))/ ///
			sum(HHI_Thirds :== 3) > 1e-6))) exit(_error("APE calculation error 9."))
		(APEmat_bP1, APEmat_bP2, APEmat_bP3)
   		return((APEmat_bP1, APEmat_bP2, APEmat_bP3))
	}
}


/* Calculate the marginal effects. */
APEmat = MargEff(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, ///
	TypebP, rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 0, S = .)
/* Display APEmat. A 1% increase in payment for kk will result in a 
.01*APEmat[jj, kk] increase in the fraction of cases treated by jj. */
/* mm_matlist(round(APEmat_bP1*1e2)*1e-2, ("%8.2f", "%8.2f", "%8.2f", "%8.2f", "%8.2f")) */

/* Find standard errors. */
fh = fopen("${Code}/Estimates/SEDraws${Name}", "rw")
D = fgetmatrix(fh)
SEmat = MargEff(FE, WWA, WWB, Ins, NonResp, RegNum, Year, HHI_Thirds, ///
	TypebP, rshpaFE, rshpaW, rshpaIns, rshpbFE, rshpbW, rshpbP, rshpSig, SE = 1, D)

FracActual = rows(WWA)^-1*(sum(Procedure:==1), sum(Procedure:==2), sum(Procedure:==3), ///
	sum(Procedure:==4), sum(Procedure:==5))
diagonal(SEmat[1..5, 1..5])':/FracActual
/* diagonal(SEmat[1..5, 6..10])':/FracActual */
diagonal(SEmat[1..5, 11..15])':/FracActual

/* If you change the log price of k by 1, then the number of j-procedures increases by
ScaleAPE_bP[j, k] percent.  */
ScaleAPE_bP1 = 100*APEmat_bP1:/FracActual'
round(ScaleAPE_bP1)*1e-2
if (TypebP == 1) ScaleAPE_bP2 = 100*APEmat_bP2:/FracActual'
diagonal(round(ScaleAPE_bP1)*1e-2)'
if (TypebP == 1) diagonal(round(ScaleAPE_bP2)*1e-2)'


else 0
end


