/* This runs the gibbs sampler repeatedly to find standard errors of the estimated 
parameters, taking into account the estimation error in the first stage. */

clear all
macro drop _all
version 12.1
set more off
set matsize 7500
set maxvar 7500
/* ssc install moremata */
set type double, perm
capture log close
global Data /disk/agedisk2/medstat.work/dcoey/Data
global Code /disk/agedisk2/medstat.work/dcoey/Code
do ${Code}/DefineGlobals.do
global BSGS 1 /* This is 0 if finding the point estimates for the parameters estimated by 
Gibbs sampling (as in Model, and 1 if bootstrapping standard errors for those 
parameters (as here). */
global OnlyaIns 0 /* This is 1 if the only thing that changes from one round to another 
is the value of aIns. If it is 0, the estimation data is also resampled. */
set seed ${seed}
log using ${Code}/Logs/StandardErrors, replace
qui do ${Code}/Elixhauser.do /* This defines the ${Elixhauser} macro. */
qui do ${Code}/${ModelPrograms}.do
qui do ${Code}/${ModelFunctions}.do

program define SE
	qui forvalues RepNum = `1'/`2'{
		noi di "Bootstrap draw number `RepNum'"
		global ngs 1000 /* Override the ngs setting from DefineGlobals. */
		if (${OnlyaIns} == 0) use ${Data}/BSData/BSData`RepNum', clear
		else use ${Data}/${EstSample}, clear
		qui do ${Code}/${SetUp}.do
		qui include ${Code}/${StartingValues}.do /* Using the aIns estimates from the 
			resampled data. "include" used here so that the RepNum macro can be 
			referenced inside StartingValues. */
		qui do ${Code}/${InitializeGibbsSampler}.do /* This sets up some of the matrices 		
			the Gibbs sampler uses. */
		include ${Code}/${GibbsSampler}.do
		timer off `RepNum'
		noi timer list
		timer clear
	}
end

SE 1 10 
SE 11 20
SE 21 30
SE 31 40
SE 41 ${BSReps} 

/* mata: 
GetEst(aFE = ., aW = ., bFE = ., bW = ., bP = ., Sig = ., TypebP = ., ///
	"$Code/Estimates", "${Name}_BS1")	
DrawsaFE = J(${BSReps}, cols(vec(aFE)'), .)
DrawsaW  = J(${BSReps}, cols(vec(aW)'), .)
DrawsbFE = J(${BSReps}, cols(vec(bFE)'), .)
DrawsbW  = J(${BSReps}, cols(vec(bW)'), .)
DrawsbP  = J(${BSReps}, cols(vec(bP)'), .)
DrawsSig = J(${BSReps}, cols(vech(Sig)'), .)
*/

/*forvalues RepNum =  1/${BSReps}{
	mata: GetEst(aFE = ., aW = ., bFE = ., bW = ., bP = ., Sig = ., TypebP = ., ///
	"$Code/Estimates", "n2000t0rSomeEmrgs888pi1ci1o10d0cl1st0_BS`RepNum'_OnlyaIns0")	
 	mata: DrawsaFE[`RepNum', .] = vec(aFE)'
	mata: DrawsaW[`RepNum', .] = vec(aW)'
	mata: DrawsbFE[`RepNum', .] = vec(bFE)'
	mata: DrawsbW[`RepNum', .] = vec(bW)' 
	mata: DrawsbP[`RepNum', .] = vec(bP)'
 	mata: DrawsSig[`RepNum', .] = vech(Sig)' 
}
*/


mata: dbp = J(${BSReps}, 1, .)
forvalues ii = 1/$BSReps{
	mata: GetEst(aFE = ., aW = ., bFE = ., bW = ., bP = ., Sig = ., TypebP = ., ///			
		"$Code/Estimates/BootstrappedGibbsSamplerResults", /// 
		"n5000t0rSomeEmrgs10pi1ci1o10d0cl1st0_BS`ii'_OnlyaIns0")	
	mata: dbp[`ii'] = bP
}

mata: variance(dbp)^.5
	

variance(DrawsbP)^.5
log close
exit
